from __future__ import annotations

from tcsoa.gen.BusinessObjects import Cfg0AbsAdmissibility, POM_object, Cfg0ConfiguratorPerspective
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AdmissibilityData(TcBaseObj):
    """
    The output structure containing the admissibility data corresponding to the context object.
    
    :var admissibilityList: List of the admissibility objects associated with the context POM_Objects.
    :var contextObject: The context object for which the admissibility information is being returned.
    """
    admissibilityList: List[Cfg0AbsAdmissibility] = ()
    contextObject: POM_object = None


@dataclass
class AdmissibilityOutput(TcBaseObj):
    """
    The output structure containing the admissibility data corresponding to one input entry.
    
    :var admissibilityDataList: The list of admissibility data associated with the context Business Object.
    :var index: The index for corresponding input structure.
    """
    admissibilityDataList: List[AdmissibilityData] = ()
    index: int = 0


@dataclass
class GetAdmissibilityInputStruct(TcBaseObj):
    """
    The input structure for the getAdmissibility operation containing the contexts and perspective instance with
    RevisionRule.
    
    :var configPerspective: The Cfg0ConfiguratorPerspective instance to specify the context and revision rule. This
    parameter can be NULL if the operation parameter for the perspective is not NULL.
    :var contextObjects: The list of context business objects having configurator context as defined by
    configPrespective.
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    contextObjects: List[POM_object] = ()


@dataclass
class GetAdmissibilityResponse(TcBaseObj):
    """
    Response structure for getAdmissibility operation.
    
    :var admissibilityOutputList: The list of admissibility information output structures for each input.
    :var serviceData: Service data to return partial errors.
    """
    admissibilityOutputList: List[AdmissibilityOutput] = ()
    serviceData: ServiceData = None



from tcsoa.gen.ActiveWorkspaceVis._2015_03.services import DataManagementService as imp0
from tcsoa.base import TcService


class DataManagementService(TcService):
    createLaunchInfo = imp0.createLaunchInfo



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AttributeDetailsStruct(TcBaseObj):
    """
    The structure specifyies the attributes and its value.
    
    :var attributeName: The string representing the name of the valid attribute which needs to be edited.
    :var attributeValueDetails: The structure specifying details of the attribute value.
    """
    attributeName: str = ''
    attributeValueDetails: Data = None


@dataclass
class ObjectAttributesInput(TcBaseObj):
    """
    The input structure contains object(s) to be edited and the details of the attributes or relations which need to be
    edited.
    
    :var object: The business object representing the BOM line. Attributes of the objects attached to this BOM line
    will be edited. 
    :var attachedObject: The business object attached to the BOM line. Attributes of this object are edited.
    :var attributeDetails: List of the structure attributeDetailsStruct specifying the attributes and their value.
    """
    object: BusinessObject = None
    attachedObject: BusinessObject = None
    attributeDetails: List[AttributeDetailsStruct] = ()


@dataclass
class SyncStudyInput(TcBaseObj):
    """
    The Mfg0BvrStudy objects to synchronize and the direction to synchronize (to/from the study).
    
    :var study: The Mfg0BvrStudy objects to synchronize
    :var direction: The direction to synhronize (true = from BOP, false = to BOP)
    """
    study: BusinessObject = None
    direction: bool = False


@dataclass
class SyncStudyResponse(TcBaseObj):
    """
    The FMS file ticket to the log file for the study synchronization.
    
    :var logFileTicket: The fmd ticket to the log file
    :var serviceData: Standard Service Data
    """
    logFileTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class Data(TcBaseObj):
    """
    The structure specifies details of the attribute value.
    
    :var dataType: Type of the data. Valid types are "Boolean", "Character", "Integer", "Double", "String", "Tag" and
    "Date". Corresponding list in this structure need to be populated based on the data type string.
    :var boolAttributes: The list of Boolean values.
    :var charAttributes: The string representing the list of characters. Each character in the string is a value of the
    attribute.
    :var integerAttributes: The list of integer values.
    :var doubleAttributes: The list of double values.
    :var stringAttributes: The list of string values.
    :var tagAttributes: The list of business objects.
    :var dateAttributes: The list of dates.
    """
    dataType: str = ''
    boolAttributes: List[bool] = ()
    charAttributes: str = ''
    integerAttributes: List[int] = ()
    doubleAttributes: List[float] = ()
    stringAttributes: List[str] = ()
    tagAttributes: List[BusinessObject] = ()
    dateAttributes: List[datetime] = ()



from __future__ import annotations

from tcsoa.gen.Classification._2011_12.Classification import ClassificationInfoResponse
from typing import List
from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def getClassificationObjectInfo(cls, icoUids: List[str], attributeIds: List[int], getOptimizedValues: bool, fetchDescriptor: bool, locale: str) -> ClassificationInfoResponse:
        """
        Provides detailed information on classification objects based on their unique identifiers (UID). A
        classification object is also called an ICO
        
        Use cases:
        When user needs to get details of a classification object (ICO).  These details include the classification
        class to which ICO belongs, the unit system, ICO ID, various ICO attributes and their values and property
        descriptor for these attributes
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='getClassificationObjectInfo',
            library='Classification',
            service_date='2011_12',
            service_name='Classification',
            params={'icoUids': icoUids, 'attributeIds': attributeIds, 'getOptimizedValues': getOptimizedValues, 'fetchDescriptor': fetchDescriptor, 'locale': locale},
            response_cls=ClassificationInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class CreateIn(TcBaseObj):
    """
    Input for create operation including unique client identifier
    
    :var clientId: Unique client identifier
    :var context: The context for the new created object
    :var target: The target  to connect to
    :var relationName: The name of the relation to use to connect to the target. If the string is empty then the
    relation defined as default will be used.
    :var data: Input data for create operation
    """
    clientId: str = ''
    context: BusinessObject = None
    target: BusinessObject = None
    relationName: str = ''
    data: CreateInput = None


@dataclass
class CreateInput(TcBaseObj):
    """
    CreateInput structure used to capture the inputs required for creation of a business object. This is a recursive
    structure containing the CreateInput(s) for any secondary (compounded) objects that might be created (e.g Item also
    creates ItemRevision and ItemMasterForm, etc.)
    
    :var type: Business Object type name
    :var stringProps: Map containing string property values
    :var boolArrayProps: Map containing string array property values
    :var dateProps: Map containing DateTime property values
    :var dateArrayProps: Map containing DateTime array property values
    :var tagProps: Map containing string property values
    :var tagArrayProps: Map containing string array property values
    :var compoundCreateInput: CreateInput for compounded objects
    :var stringArrayProps: Map containing string array property values
    :var doubleProps: Map containing string property values
    :var doubleArrayProps: Map containing string array property values
    :var floatProps: Map containing string property values
    :var floatArrayProps: Map containing string array property values
    :var intProps: Map containing string property values
    :var intArrayProps: Map containing string array property values
    :var boolProps: Map containing string property values
    """
    type: str = ''
    stringProps: StringMap = None
    boolArrayProps: BoolVectorMap = None
    dateProps: DateMap = None
    dateArrayProps: DateVectorMap = None
    tagProps: TagMap = None
    tagArrayProps: TagVectorMap = None
    compoundCreateInput: CreateInputMap = None
    stringArrayProps: StringVectorMap = None
    doubleProps: DoubleMap = None
    doubleArrayProps: DoubleVectorMap = None
    floatProps: FloatMap = None
    floatArrayProps: FloatVectorMap = None
    intProps: IntMap = None
    intArrayProps: IntVectorMap = None
    boolProps: BoolMap = None


@dataclass
class CreateOut(TcBaseObj):
    """
    Output for create operation including unique client identifier
    
    :var clientId: Unique client identifier
    :var objects: Vector of tags representing objects that were created
    """
    clientId: str = ''
    objects: List[BusinessObject] = ()


@dataclass
class CreateResponse(TcBaseObj):
    """
    Response object for create operation
    
    :var output: Vector of output objects representing objects that were created
    :var serviceData: Service data including partial errors that are mapped to the client id
    """
    output: List[CreateOut] = ()
    serviceData: ServiceData = None


@dataclass
class DisconnectInput(TcBaseObj):
    """
    Input for disconnect operation
    
    :var clientId: Unique client identifier
    :var object: The object to disconnect
    :var disconnectFrom: The object to disconnect from
    :var relationName: The relation to disconnect in case 2 objects are connected by more than one relation
    """
    clientId: str = ''
    object: BusinessObject = None
    disconnectFrom: BusinessObject = None
    relationName: str = ''


"""
CreateInputMap
"""
CreateInputMap = Dict[str, List[CreateInput]]


"""
DateMap
"""
DateMap = Dict[str, datetime]


"""
DateVectorMap
"""
DateVectorMap = Dict[str, List[datetime]]


"""
DoubleMap
"""
DoubleMap = Dict[str, float]


"""
DoubleVectorMap
"""
DoubleVectorMap = Dict[str, List[float]]


"""
FloatMap
"""
FloatMap = Dict[str, float]


"""
FloatVectorMap
"""
FloatVectorMap = Dict[str, List[float]]


"""
IntMap
"""
IntMap = Dict[str, int]


"""
IntVectorMap
"""
IntVectorMap = Dict[str, List[int]]


"""
StringMap
"""
StringMap = Dict[str, str]


"""
BoolMap
"""
BoolMap = Dict[str, bool]


"""
StringVectorMap
"""
StringVectorMap = Dict[str, List[str]]


"""
BoolVectorMap
"""
BoolVectorMap = Dict[str, List[bool]]


"""
TagMap
"""
TagMap = Dict[str, BusinessObject]


"""
TagVectorMap
"""
TagVectorMap = Dict[str, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BidPackageRevision, BidPackage, Vendor, Part, VendorRevision, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class BidPackageProps(TcBaseObj):
    """
    The information about each BidPackage to be processed is provided by way of the 'BidPackageProps' data structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify the Partial Errors associated
    with this 'BidPackageProps' input.
    :var itemId: Id to create BidPackage, generated if the string is empty.
    :var name: Name for the BidPackage. Default name generated if left empty.
    :var type: Object Type to be created. It is BidPackage.
    :var revId: Revision Id to create BidPackageRevision. Generated if left empty.
    :var description: Description of the object to be created.
    """
    clientId: str = ''
    itemId: str = ''
    name: str = ''
    type: str = ''
    revId: str = ''
    description: str = ''


@dataclass
class CreateVendorsOutput(TcBaseObj):
    """
    The information about each Vendor object output after creation is provided by way of the 'CreateVendorsOutput' data
    structure.
    
    :var clientId: A unique string supplied by the caller.
    :var vendor: Vendor Object created or updated.
    :var vendorRev: VendorRevision Object created or Updated.
    """
    clientId: str = ''
    vendor: Vendor = None
    vendorRev: VendorRevision = None


@dataclass
class CreateVendorsResponse(TcBaseObj):
    """
    The 'CreateVendorsResponse' structure represents the output vector 'CreateVendorsOutput' and the 'ServiceData'.
    
    :var output: A list of 'CreateVendorsOutput' structures. Each of them represents a Vendor created or updated.
    :var serviceData: Standard 'ServiceData' structure. It contains partial errors and failures along with the
    clientIds.
    """
    output: List[CreateVendorsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class LineItemProps(TcBaseObj):
    """
    The information about each BidPackageLineItem to be processed is provided by way of the 'LineItemProps' data
    structure.
    
    :var name: Name of the BidPackageLineItem to be created.
    :var description: Description of the BidPackageLineItem Object.
    :var quote: Model Object for Quote to be added to BidPackageLineItem.
    :var liccname: Name of the LineItemConfigContext to be created.
    :var liccdesc: Description for the 'LineItemConfigContext' object to be created.
    :var partid: Id of the part to be associated with BidPackageLineItem.
    :var viewtype: 'PSViewType' to be associated with LineItemConfigContext.
    :var quantity: Quantity to be created for BidPackageLineItem.
    :var revRule: Revision rule to be associated with LineItemConfigContext.
    :var varRule: Variant rule to be associated with LineItemConfigContext.
    :var closureRule: Closure rule to be associated with LineItemConfigContext.
    """
    name: str = ''
    description: str = ''
    quote: BusinessObject = None
    liccname: str = ''
    liccdesc: str = ''
    partid: str = ''
    viewtype: str = ''
    quantity: int = 0
    revRule: str = ''
    varRule: str = ''
    closureRule: str = ''


@dataclass
class VendorPartProperties(TcBaseObj):
    """
    The information about each VendorPart to be processed is provided by way of the 'VendorPartProperties' data
    structure.
    
    :var partId: Id for part to be created, mandatory for ManufacturerPart objects but generated for CommercialPart
    objects if the string is empty. Output PartId in case of ManufacturerPart will contain the provided partId and
    provided VendorId.
    :var name: Name of the Part object to be created.
    :var uom: Unit of Measure for Model Object.
    :var makebuy: Make Buy Value for Part. The value could be either Make or Buy.
    :var clientId: A unique string supplied by the caller. This ID is used to identify the Partial Errors associated
    with this 'VendorPartProperties' input.
    :var type: Object type to be created. Only ManufacturerPart and CommercialPart are valid.
    :var revId: Revision Id specified for create. Generated if the string is empty.
    :var description: Description of the VendorPart.
    :var vendorid: Vendor Id to be associated with Part. Vendor Id is optional for CommercialPart but mandatory for
    ManufacturerPart.
    :var commercialpartid: Id of CommercialPart to be associated with ManufacturerPart.
    :var commercialpartrevid: CommercialPartRevision Id to be associated with ManufacturerPart. If blank,
    ManufacturerPart is attached with the CommercialPart instead.
    :var isDesignReq: Flag to decide if the design is required.
    """
    partId: str = ''
    name: str = ''
    uom: BusinessObject = None
    makebuy: int = 0
    clientId: str = ''
    type: str = ''
    revId: str = ''
    description: str = ''
    vendorid: str = ''
    commercialpartid: str = ''
    commercialpartrevid: str = ''
    isDesignReq: bool = False


@dataclass
class VendorProperties(TcBaseObj):
    """
    The information about each Vendor to be processed is provided by way of the 'VendorProperties' data structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify the Partial Errors associated
    with this 'VendorProperties' input.
    :var itemId: Item Id of the Vendor. Generated if the string is empty.
    :var name: Name of the Vendor. Default name is generated if kept empty.
    :var type: Object type to be created.
    :var revId: VendorRevision Id specified for create. Generated if the string is empty.
    :var description: Description of the Vendor.
    :var roleType: Type of VendorRole. It can be Manufacturer,Supplier, Distributor or blank.
    :var certifiStatus: Vendor Certification status like Gold,Silver etc.
    :var vendorStatus: Vendor status like Preferred,Approved etc.
    """
    clientId: str = ''
    itemId: str = ''
    name: str = ''
    type: str = ''
    revId: str = ''
    description: str = ''
    roleType: str = ''
    certifiStatus: str = ''
    vendorStatus: str = ''


@dataclass
class CreateBidPacksOutput(TcBaseObj):
    """
    The information about each BidPackage object output after creation is provided by  way of the 'CreateVendorsOutput'
    data structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify the Partial Errors associated
    with input.
    :var bidPackage: Model Object pointing to the BidPackage created.
    :var bidPackageRev: Model Object poinitng to the BidPackageRevision created.
    """
    clientId: str = ''
    bidPackage: BidPackage = None
    bidPackageRev: BidPackageRevision = None


@dataclass
class CreateBidPacksResponse(TcBaseObj):
    """
    The 'CreateBidPacksResponse' structure represents the output vector 'CreateBidPacksOutput' and standard Teamcenter
    'ServiceData' structure instance.
    
    :var output: A list of 'CreateBidPacksOutput' structures. Each entry represents a BidPackage created or updated.
    :var serviceData: Standard 'ServiceData' structure. It contains typical potential errors as mentioned in throws and
    failures along with the clientIds.
    """
    output: List[CreateBidPacksOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateVendorPartsOutput(TcBaseObj):
    """
    The information about each VendorPart object output after creation is provided by way of the
    'CreateVendorPartsOutput' data structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify the Partial Errors associated
    with input.
    :var vendorPart: Model Object pointing to the VendorPart created.
    :var vendorPartRev: Model Object poinitng to the VendorPart revision created.
    """
    clientId: str = ''
    vendorPart: Part = None
    vendorPartRev: ItemRevision = None


@dataclass
class CreateVendorPartsResponse(TcBaseObj):
    """
    The 'CreateVendorPartsResponse' structure represents the list of 'CreateVendorPartsOutput' and Standard Teamcenter
    'ServiceData'.
    
    :var output: Output is list of 'CreateVendorPartsOutput' data structures, each of them representing a new
    VendorPart created.
    :var serviceData: Standard 'ServiceData' structure. It contains partial errors and failures along with the
    clientIds.
    """
    output: List[CreateVendorPartsOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.IcsAw._2019_06.Classification import ClassifyCommandVisibilityInfoResp
from typing import List
from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def getClassificationCmdVisibilityInfo(cls, theWSOs: List[BusinessObject]) -> ClassifyCommandVisibilityInfoResp:
        """
        Finds classification command visibility information for traditional and Classification Standard Taxonomy (CST)
        for the list of workspace objects.
        If a given WorkspaceObject type is or dependent type is not present in ICS_classifiable_types preference or the
        user does not have permission to classify the workspace object then this operation returns false for the
        corresponding workspace object in the response.
        If a given workspace object type is or dependent type is present in ICS_classifiable_types and the user has the
        permission to classify the workspace object then this operation returns true for the corresponding workspace
        object in the response.
        
        Use cases:
        This operation could be used when user needs to find classification command visibility information, for given
        workspace objects in both traditional and Classification Standard Taxonomy (CST). 
        Each time a workspace object is selected for classify operation, this operation will evaluate the "Add" command
        visibility button in the classification tab based on workspace object type and user&rsquo;s privileges to
        classify the given WorkspaceObject.
        """
        return cls.execute_soa_method(
            method_name='getClassificationCmdVisibilityInfo',
            library='Internal-IcsAw',
            service_date='2019_06',
            service_name='Classification',
            params={'theWSOs': theWSOs},
            response_cls=ClassifyCommandVisibilityInfoResp,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanSubscription
from tcsoa.gen.Internal.Notification._2015_10.SubscriptionManagement import ModifySubscriptionTemplateInput, SubscriptionTemplateResponse, GetAttachEventDataResponse2, SubscriptionTemplate, SaveAsSubscriptionInputData
from typing import List
from tcsoa.gen.Notification._2014_10.SubscriptionManagement import SubscriptionsResponse
from tcsoa.base import TcService
from tcsoa.gen.Internal.Notification._2015_10.MessageManagement import UnreadMessages


class MessageManagementService(TcService):

    @classmethod
    def getUnreadMessages(cls) -> UnreadMessages:
        """
        Get the Fnd0Message object UIDs of the unread messages for the current user. This operation is supported only
        in a 4Tier environment with a asyncronous messaging system installed. If called from a 2Tier client, an empty
        list will be returned.
        
        Use cases:
        The Active Workspace client calls the getUnreadMessages operation in a polling fashion to get the number of
        unread messages for the current user. This number is displayed in the client UI. When the user selects the
        message icon, the client will query the Teamcneter server for the full list of messages complete with details.
        
        Exceptions:
        >78040: The asynchronous messaging system is not installed or configured correctly. Please contact the system
        administrator to configure the Teamcenter server for asynchronous messaging support.
        """
        return cls.execute_soa_method(
            method_name='getUnreadMessages',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='MessageManagement',
            params={},
            response_cls=UnreadMessages,
        )

    @classmethod
    def initializeAsynchMessaging(cls) -> str:
        """
        Register the current user with the Broker , and initialize the list of unread messages for the current user in
        the Broker. The Broker is a component installed as part of the Pool Manager, and marshalls events from the
        Teamcenter server (result of this service operation) and the Subscription Manager  to the Message Cache module
        in the the Teamcenter web tier.
        
        Exceptions:
        >78043 Initialization of the aysnchronous messageing system failed. Please contact the system administrator.
        """
        return cls.execute_soa_method(
            method_name='initializeAsynchMessaging',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='MessageManagement',
            params={},
            response_cls=str,
        )


class SubscriptionManagementService(TcService):

    @classmethod
    def modifySubscriptionTemplates(cls, inputs: List[ModifySubscriptionTemplateInput]) -> SubscriptionTemplateResponse:
        """
        The modifySubscriptionTemplate operation modifies Fnd0SubscriptionTemplate objects.  Subscription templates are
        used to specify predefined criteria and closure rules during the creation of an Fnd0IndirectSubscription object.
        
        Use cases:
        The user wishes to modify an existing Fnd0SubscriptionTemplate instance object.  The user can pass modified
        values such as the name, description, subscribed object criteria, target object criteria or the closure rule. 
        The values from the input will be set on the specified Fnd0SubscriptionTemplate objects.
        """
        return cls.execute_soa_method(
            method_name='modifySubscriptionTemplates',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='SubscriptionManagement',
            params={'inputs': inputs},
            response_cls=SubscriptionTemplateResponse,
        )

    @classmethod
    def saveAsSubscription(cls, input: SaveAsSubscriptionInputData) -> SubscriptionsResponse:
        """
        The saveAsSubscription operation unsubscribes the current user from the input subscription and creates another
        subscription using the input subscription. The current user will be the subscriber, and the input name will be
        the name of the new subscription.
        
        Use cases:
        A user receives a notification for a subscription, but the user wishes to make a modification to the
        subscription and receive notifications on the basis of a new criterion. The user then performs
        saveAsSubscription and provides a new name to the subscription.
        """
        return cls.execute_soa_method(
            method_name='saveAsSubscription',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='SubscriptionManagement',
            params={'input': input},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def unsubscribe(cls, subscriptions: List[ImanSubscription]) -> SubscriptionsResponse:
        """
        The unsubscribe operation takes a list of ImanSubscription objects as input and unsubscribes the user from the
        subscriptions. The current user is removed from the notification list, causing the user to stop receiving
        notification for that subscription.
        
        Use cases:
        A user who is in the notification list of a subscription receives a notification. The user can choose to
        unsubscribe from receiving these notifications by calling unsubscribe.
        """
        return cls.execute_soa_method(
            method_name='unsubscribe',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='SubscriptionManagement',
            params={'subscriptions': subscriptions},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def createSubscriptionTemplates(cls, inputs: List[SubscriptionTemplate]) -> SubscriptionTemplateResponse:
        """
        The createSubscriptionTemplates operation creates multiple Fnd0SubscriptionTemplate objects which are then used
        in the creation of indirect subscriptions. The traversal rule, primary object criteria and the target object
        criteria is used to process the Fnd0IndirectSubscription objects.
        
        Use cases:
        An admin user creates a Closure Rule using the PLMXML UI. Then using the Subscription Template creation UI in
        the Subscription admin application, the admin user picks a subscribed object type and defines the criteria on
        it, if any. Then a target object type is selected and its criteria are defined, if any. Then the user selects
        the TIE closure rule created and saves the template upon which the createSubscriptionTemplate operation is
        called. The template created can then be used for the creation of an Fnd0IndirectSubscription object.
        """
        return cls.execute_soa_method(
            method_name='createSubscriptionTemplates',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='SubscriptionManagement',
            params={'inputs': inputs},
            response_cls=SubscriptionTemplateResponse,
        )

    @classmethod
    def getApplicableSubscriptionTemplates(cls, subscribedObject: BusinessObject) -> SubscriptionTemplateResponse:
        """
        The getApplicableSubscriptionTemplates operation retrieves the applicable Fnd0SubscriptionTemplate objects
        based on the subscribed object data.
        """
        return cls.execute_soa_method(
            method_name='getApplicableSubscriptionTemplates',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='SubscriptionManagement',
            params={'subscribedObject': subscribedObject},
            response_cls=SubscriptionTemplateResponse,
        )

    @classmethod
    def getAttachEventData2(cls, target: BusinessObject) -> GetAttachEventDataResponse2:
        """
        The getAttachEventData2 operation retrieves the lists of relation objects and attachment types applicable on
        the target object. When an event is subscribable, that means subscriptions can be created for that event. The
        attribute criteria for the subscription can be populated on the basis of the data returned using this operation.
        
        Use cases:
        The client selects a subscribable business object that will be the target object of a subscription. The client
        is returned with a list of event types using the getSubscriptionInput operation. Then, if the user selects
        attach event, the lists of relation objects and attachment types applicable on the target object are returned.
        """
        return cls.execute_soa_method(
            method_name='getAttachEventData2',
            library='Internal-Notification',
            service_date='2015_10',
            service_name='SubscriptionManagement',
            params={'target': target},
            response_cls=GetAttachEventDataResponse2,
        )



from __future__ import annotations

from tcsoa.base import TcService
from tcsoa.gen.Internal.ProjectManagement._2012_02.ScheduleManagement import TranslatorResponseScheduleModifyContainer


class ScheduleManagementService(TcService):

    @classmethod
    def translateFive(cls, scheduleUid: str) -> TranslatorResponseScheduleModifyContainer:
        """
        this is a non functional internal soa that would never be called. It is a tagging
        method for  translatorResponseScheduleModifyContainer structure which is used internally
        to transfer objects between C++ and Java codes. Without the tagging the soa generate
         command would fail.
        """
        return cls.execute_soa_method(
            method_name='translateFive',
            library='Internal-ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'scheduleUid': scheduleUid},
            response_cls=TranslatorResponseScheduleModifyContainer,
        )



from __future__ import annotations

from tcsoa.base import TcService


class SimulationProcessManagementService(TcService):

    @classmethod
    def notifyUser(cls, message: str, users: str, groups: str) -> bool:
        """
        This internal operation will send email notification to all the configured users after the completion of a
        simulation process launch. This notification will contain the details of the launch relative to its success or
        failure and also contain information about newly created objects (Item, ItemRevision, Dataset) if any.
        To use this operation, the user should have either a simulation_author or rtt_author license.
        """
        return cls.execute_soa_method(
            method_name='notifyUser',
            library='Internal-Cae',
            service_date='2009_10',
            service_name='SimulationProcessManagement',
            params={'message': message, 'users': users, 'groups': groups},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.Administration._2015_07.UserManagement import CreateOrUpdateUserResponse, CreateOrUpdateUserInputs
from typing import List
from tcsoa.base import TcService


class UserManagementService(TcService):

    @classmethod
    def createOrUpdateUser(cls, userInputs: List[CreateOrUpdateUserInputs]) -> CreateOrUpdateUserResponse:
        """
        This operation creates or updates User objects with given properties. A new User object will be created if it
        does not already exist, otherwise existing User object will be updated. This operation requires system
        administration privilege.
        
        If the Person object given as part of input does not exist it will be created. If it exist the newly created
        User object will point to given Person object.
        
        If not specified in user input, the license level will be author  and user status will be active by default in
        user creation.
        
        License Server and License Bundle on User can be updated by giving license_server and license_bundle as
        property names in userPropertyMap of CreateOrUpdateUserInputs.
        
        
        Use cases:
        Use Case 1: Create User with set of properties.
        Use Case 2: Create User with additional properties.
        Use Case 3: Update existing User&rsquo;s properties.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateUser',
            library='Administration',
            service_date='2015_07',
            service_name='UserManagement',
            params={'userInputs': userInputs},
            response_cls=CreateOrUpdateUserResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SearchDynamicIPAsOutput(TcBaseObj):
    """
    Object containing parent process or study and its dynamic in-process assembly objects.
    
    :var bopLine: Parent process Mfg0BvrProcess or study Mfg0BvrShdStudy object.
    :var dipaLines: A list of dynamic in-process assembly Mfg0BvrDynamicIPA objects.
    """
    bopLine: BusinessObject = None
    dipaLines: List[BusinessObject] = ()


@dataclass
class SearchDynamicIPAsResponse(TcBaseObj):
    """
    Response object for searchDynamicIPAs operation.
    
    :var output: List of objects containing parent process or study and its dynamic in-process assembly objects.
    :var serviceData: Standard service data.
    """
    output: List[SearchDynamicIPAsOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Discipline
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDisciplineResponse(TcBaseObj):
    """
    This structure is the object returned by this operation. It holds the Discipline object found and ServiceData
    object.
    
    :var discipline: The discipline object found with the given name.
    :var serviceData: The object which holds the possible error in the search of the discipline.
    """
    discipline: Discipline = None
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class Arg(TcBaseObj):
    """
    Arg
    
    :var val: Argument value
    :var structure: Structure if argument is structure
    :var array: Array if argument is array
    """
    val: str = ''
    structure: List[Structure] = ()
    array: List[Array] = ()


@dataclass
class Array(TcBaseObj):
    """
    Array
    
    :var entries: Array entries
    """
    entries: List[Entry] = ()


@dataclass
class InvokeICTMethodResponse(TcBaseObj):
    """
    Response of invokeICTMethod method
    
    :var output: Output arguments
    :var serviceData: service data
    """
    output: List[Arg] = ()
    serviceData: ServiceData = None


@dataclass
class Structure(TcBaseObj):
    """
    structure
    
    :var args: List of arguments
    """
    args: List[Arg] = ()


@dataclass
class Entry(TcBaseObj):
    """
    Array entry
    
    :var val: Entry value
    :var structure: Structure if entry is a structure
    :var array: Array if entry is an array
    """
    val: str = ''
    structure: List[Structure] = ()
    array: List[Array] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, Group
from tcsoa.base import TcService
from tcsoa.gen.Internal.Administration._2016_09.VolumeManagement import VolumeInfoResponse


class VolumeManagementService(TcService):

    @classmethod
    def getAccessibleVolumes(cls, user: User, group: Group) -> VolumeInfoResponse:
        """
        This operation returns the  list of accessible volumes, default volume and default local volume  for the given 
        user and group. If the preference TC_allow_inherited_group_volume_access is set to a non-zero value, the
        operation considers access granted to volumes on all parent level groups up the chain and uses this information
        for computing the accessibleVolumes, defaultVolume and defaultLocalVolume.
        """
        return cls.execute_soa_method(
            method_name='getAccessibleVolumes',
            library='Internal-Administration',
            service_date='2016_09',
            service_name='VolumeManagement',
            params={'user': user, 'group': group},
            response_cls=VolumeInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2010_04.BOMMarkup import ApplyBOMMarkupParam, CreateBOMMarkupResponse, CreateBOMMarkupParam
from tcsoa.gen.BusinessObjects import BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class BOMMarkupService(TcService):

    @classmethod
    def savePendingEditsAsMarkup(cls, window: BOMWindow) -> CreateBOMMarkupResponse:
        """
        All pending edits of the specified window are persisted as Markups.
        NOTE: not implemented
        
        Use cases:
        The user must first have established a single-structure editing session in a BOMWindow, with the ability to
        create pending edits enabled. If the user wants to keep the pending edits of the current BOMWindow after the
        conclusion of this session, this method will create BOMMarkups corresponding to the pending edits and store
        them in the database.
        """
        return cls.execute_soa_method(
            method_name='savePendingEditsAsMarkup',
            library='Internal-StructureManagement',
            service_date='2010_04',
            service_name='BOMMarkup',
            params={'window': window},
            response_cls=CreateBOMMarkupResponse,
        )

    @classmethod
    def createBOMMarkup(cls, input: List[CreateBOMMarkupParam]) -> CreateBOMMarkupResponse:
        """
        Creates the Markup Change objects for the specified base structures. The Markup Changes are added to active
        Markups. This operation may create a new Markup object, if no active Markup exists, and the created new Markup
        object will be attached to the BOMViewRevision object of the BOMLine. If the line has no BOMViewRevision
        object, it will create one.
        
        Note:
        - If a line is marked as to be removed, no additional Markup Change can be created for the line.
        - If a line already has a Markup Change, the line cannot be marked for removal.
        - If a line already has a 'PROPERTY' Markup Change for exactly the same property, the old property value Markup
        Change will be overridden by the new proposed property value change.
        - If a line already has a Markup Change for replace, it cannot have another 'REPLACE' Markup Change or a
        'SUBSTITUTE' Markup Change.
        - If a line already has a Markup Change for substitute change to add a substitute or remove a substitute, the
        line cannot have a 'REPLACE' Markup Change.
        - If a line already has a Markup Change for substitute change to add a substitute or remove a substitute, the
        line cannot have another exactly the same substitute Markup Change.
        
        
        
        Use cases:
        - If user wants to create an 'ADD' Markup Change to a line, user can call the operation with the BOMLine object
        and the Item/Item Revision to be added. If user wants have initial value to find number, quantity and
        occurrence type properties for the 'ADD' Markup Changes, user will need to add property Markup Changes for
        these properties immediately after the 'ADD' Markup Change in the input of the operation. 
        - If user wants to make a Markup Change to add a substitute, user can call the operation by passing in the
        selected line, Item/Item Revision object as input object, and its BOMView object as context.
        - If user wants to make a Markup Change to change legacy variant condition, user can call the operation by
        passing the selected line,  the variant expression object of the variant condition as input object,  and
        "bl_condition_tag" as the property name to create the Markup Change object.
        
        """
        return cls.execute_soa_method(
            method_name='createBOMMarkup',
            library='Internal-StructureManagement',
            service_date='2010_04',
            service_name='BOMMarkup',
            params={'input': input},
            response_cls=CreateBOMMarkupResponse,
        )

    @classmethod
    def applyBOMMarkup(cls, input: List[ApplyBOMMarkupParam]) -> ServiceData:
        """
        This operation applies  proposed MarkupChanges that were stored in the active Markup for the specified Bill of
        Material, if the 'evaluate' flag is 'false'. If the 'evaluate' flag is 'true', the write access to perform the
        save is checked and no modification is actually attempted. The objects that the user could not modify are
        returned in the partial errors. If the 'recursive' flag is set, then all active Markups in the structure under
        the specified BOM are evaluated and/or saved. Processing continues through the structure, and all errors
        encountered are reported in the output. Applied Markup changes will be set to 'applied' status and applied
        Markup will be set to not-active status. The Markup can be tracked from the BOMViewRevision object where it is
        attached.
        
        Use cases:
        - When the structure has MarkupChanges, user may want to check if he can apply the Changes to the structure by
        invoking the operation to evaluate the write access to apply the MarkupChanges. If no write access, user may
        revise the structure or ask author of the structure to apply the MarkupChanges.
        - User who has write access to the structure after MarkupChanges are approved can invoke the operation to apply
        all MarkupChanges of the structure including substructures.
        
        """
        return cls.execute_soa_method(
            method_name='applyBOMMarkup',
            library='Internal-StructureManagement',
            service_date='2010_04',
            service_name='BOMMarkup',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import RequestObject, AppInterface
from tcsoa.gen.Ai._2012_09.Ai import FindRequestsFilter, ProjectFilter, FindRequestsResponse, ProjectInfo, RequestInfo, GetRequestsInfo2Response, GetProjectsInfo2Response
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class AiService(TcService):

    @classmethod
    def getProjects(cls, filter: ProjectFilter) -> ServiceData:
        """
        get the Application Interface objects based on a optional filter.
        """
        return cls.execute_soa_method(
            method_name='getProjects',
            library='Ai',
            service_date='2012_09',
            service_name='Ai',
            params={'filter': filter},
            response_cls=ServiceData,
        )

    @classmethod
    def getProjectsInfo2(cls, projects: List[AppInterface]) -> GetProjectsInfo2Response:
        """
        return the projectInfo information for each of the supplied ApplicationInterface Objects.
        """
        return cls.execute_soa_method(
            method_name='getProjectsInfo2',
            library='Ai',
            service_date='2012_09',
            service_name='Ai',
            params={'projects': projects},
            response_cls=GetProjectsInfo2Response,
        )

    @classmethod
    def getRequestsInfo2(cls, robjects: List[RequestObject]) -> GetRequestsInfo2Response:
        """
        get details about specific RequestObjects. These include state desc,status info, custom key value pairs.
        """
        return cls.execute_soa_method(
            method_name='getRequestsInfo2',
            library='Ai',
            service_date='2012_09',
            service_name='Ai',
            params={'robjects': robjects},
            response_cls=GetRequestsInfo2Response,
        )

    @classmethod
    def setProjectsInfo(cls, infos: List[ProjectInfo]) -> ServiceData:
        """
        set the info on the ApplicationInterface Objects.
        """
        return cls.execute_soa_method(
            method_name='setProjectsInfo',
            library='Ai',
            service_date='2012_09',
            service_name='Ai',
            params={'infos': infos},
            response_cls=ServiceData,
        )

    @classmethod
    def setRequestsInfo(cls, infos: List[RequestInfo]) -> ServiceData:
        """
        method to allow caller to set the fields on the RequestObject.
        """
        return cls.execute_soa_method(
            method_name='setRequestsInfo',
            library='Ai',
            service_date='2012_09',
            service_name='Ai',
            params={'infos': infos},
            response_cls=ServiceData,
        )

    @classmethod
    def findRequests(cls, filter: FindRequestsFilter) -> FindRequestsResponse:
        """
        method to find request objects based on the input criteria.
        """
        return cls.execute_soa_method(
            method_name='findRequests',
            library='Ai',
            service_date='2012_09',
            service_name='Ai',
            params={'filter': filter},
            response_cls=FindRequestsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2019_06.OccurrenceManagement import OccurrencesData2, OccurrencesResp2, RequestPreference2, PackElementsResp, CloneContentData, UserContextState2, InfoForAddElemResp2, InfoForAddElemData2, AddObjectResp2, AddObjectData2, PackElementsData
from tcsoa.gen.Internal.ActiveWorkspaceBom._2019_06.Compare import CompareContentAsyncData2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def addObject2(cls, input: AddObjectData2) -> AddObjectResp2:
        """
        This operation adds an object to a product as specified in Awb0ProductContextInfo or parent Awb0Element and
        creates Awb0Element representing newly created object. The new object is added as a sibling to the selected
        Awb0Element if that is provided in input. The Awb0ProductContextInfo and parent Awb0Element can be retrieved
        using getOccurrences.
        
        Use cases:
        User loads the product structure in Content. The Content is used for smart navigation of occurrences in a
        structure. User selects an occurrence in the Content and clicks on Add button. This actions will add a new
        occurrence as a child or sibling of the selected occurrence.
        """
        return cls.execute_soa_method(
            method_name='addObject2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=AddObjectResp2,
        )

    @classmethod
    def cloneContent(cls, inputs: List[CloneContentData]) -> ServiceData:
        """
        This operation validates and clones content depending on the input criteria.
        
        If the clone operation type requested by the caller is not supported then the system will default to "Clone"
        operation for that line.
        The caller can define a specific naming pattern for the Item IDs for cloned structure. The default pattern can
        be defined by adding prefixes, suffixes or replacing part of the original name with a different pattern. The
        caller can also choose to allow the system to assign default ids. If input has insufficient or incorrect
        information then system will send an error in service data.
        """
        return cls.execute_soa_method(
            method_name='cloneContent',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='OccurrenceManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def cloneContentAsync(cls, inputs: List[CloneContentData]) -> None:
        """
        This operation performs asynchronous clone of the content depending on the input criteria.
        
        If the clone operation type requested by the caller is not supported then the system will default to "Clone"
        operation for that line.
        
        The caller can define a specific naming pattern for the Item IDs for cloned structure. The default pattern can
        be defined by adding prefixes, suffixes or replacing part of the original name with a different pattern. The
        caller can also choose to allow the system to assign default ids. If input has insufficient or incorrect
        information then system will send an error in service data.
        """
        return cls.execute_soa_method(
            method_name='cloneContentAsync',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='OccurrenceManagement',
            params={'inputs': inputs},
            response_cls=None,
        )

    @classmethod
    def getInfoForAddElement2(cls, getInfoForElementIn: InfoForAddElemData2) -> InfoForAddElemResp2:
        """
        This operation retreives information required for creating an Awb0Element under product specified in
        Awb0Element. The operation also returns allowable type name(s) to search existing object through Full Text
        Search. The preferred type is the parent type if that is allowed.This operation also indicates if allowable
        type is of Occurrence type.
        
        Use cases:
        &bull;    Returns the allowed child types for given parent type as Item or sub-types.
        &bull;    Returns the allowed child occurrence types for given parent type as ProductItem.
        &bull;    Returns the allowed child occurrence types for given parent type as occurrence.
        &bull;    Returns the allowed related object types for the given occurrence type.
        """
        return cls.execute_soa_method(
            method_name='getInfoForAddElement2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='OccurrenceManagement',
            params={'getInfoForElementIn': getInfoForElementIn},
            response_cls=InfoForAddElemResp2,
        )

    @classmethod
    def getOccurrences2(cls, inputData: OccurrencesData2) -> OccurrencesResp2:
        """
        Retrieves the page of configured occurrences for given the top-level product and configuration parameters as
        input. The service provides the facility to optionally filter and sort the result by additional filters and
        sorting criteria that may be provided as input. The output also contains a cursor that defines the place to
        start the next page location. The cursor must be passed back in to any subsequent call to get the next page of
        occurrences.
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException
        """
        return cls.execute_soa_method(
            method_name='getOccurrences2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=OccurrencesResp2,
        )

    @classmethod
    def packSimilarElements(cls, input: PackElementsData) -> PackElementsResp:
        """
        The operation provides the "pack" or "unpack" functionality on a set of Awb0Element objects.
        """
        return cls.execute_soa_method(
            method_name='packSimilarElements',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=PackElementsResp,
        )

    @classmethod
    def saveUserWorkingContextState2(cls, contextState: UserContextState2, requestPref: RequestPreference2) -> ServiceData:
        """
        This operation saves current user's client state information for the opened object. The saved information is
        used to establish user's working state while opening the object again.
        """
        return cls.execute_soa_method(
            method_name='saveUserWorkingContextState2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='OccurrenceManagement',
            params={'contextState': contextState, 'requestPref': requestPref},
            response_cls=ServiceData,
        )


class CompareService(TcService):

    @classmethod
    def compareContentAsync2(cls, input: CompareContentAsyncData2) -> None:
        """
        This operation performs asynchronous comparison of two contents (can also be refered to as source and target)
        and stores the output of comparision results in a dataset. The content can be Item BVR structures or a 4th
        Generation Design or BOM data contained within a Subset.
        """
        return cls.execute_soa_method(
            method_name='compareContentAsync2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2019_06',
            service_name='Compare',
            params={'input': input},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RuntimeBusinessObject, Dataset
from typing import Dict, List
from tcsoa.gen.Internal.StructureManagement._2013_05.StructureExpansionLite import RelationTypesAndCriteria
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpansionResponse3(TcBaseObj):
    """
    'ExpansionResponse3' contains a map with key as parent Fnd0BomLineLite or BOMLine and values as 'ChildLineInfo'.
    Additionally it also contains 'DatasetInfo' and UIDs of undelivered Fnd0BomLineLite objects.
    
    :var parentChildInfo: A map (business object, list of 'ChildLineInfo') the Key either a Fnd0BOMLineLite or BOMLine 
    parent line and values a list of 'ChildLineInfo' which contains child line object,' index to DatasetInfo 'and
    configuration accuracy information.
    :var datasetInfo: 'DatasetInfo' contains name of GRM relation through which underlying object of Fnd0BOMLineLite or
    BOMLine is attached to Dataset, Dataset object and list of 'NamedRefInfo'.
    :var parentUndeliveredChildrenUids: A map (business object, list of string) Parent BOMLine or Fnd0BOMLineLite and
    its children UIDs which are yet to be transferred to client.
    :var serviceData: The service data containing partial errors.
    """
    parentChildInfo: ParentLineToChildLineInfo3 = None
    datasetInfo: List[DatasetInfo3] = ()
    parentUndeliveredChildrenUids: ParentLineToChildrenLineUidValues2 = None
    serviceData: ServiceData = None


@dataclass
class NamedRefInfo3(TcBaseObj):
    """
    Contains details of named reference object of Dataset.
    
    :var name: Name of reference object in Dataset. For example, JTPart.
    :var type: The type of the named reference object.
    :var namedReference: The named reference object. Some common named reference objects are of type ImanFile and Form.
    :var originalFileName: The original_file_name attribute value of the file in case the named reference object is a
    file.
    :var fileTicket: FMS ticket used to retrieve the file.
    """
    name: str = ''
    type: str = ''
    namedReference: BusinessObject = None
    originalFileName: str = ''
    fileTicket: str = ''


@dataclass
class AbsoccDataIncludeList(TcBaseObj):
    """
    This List contains Absoccdata overrides type that would be considered during "LWB Configuration".
    
    :var absoccDataTypeInclusionList: List of AbsoccDataOverrideInfo.
    """
    absoccDataTypeInclusionList: List[AbsoccDataOverrideInfo] = ()


@dataclass
class AbsoccDataOverrideInfo(TcBaseObj):
    """
    'AbsoccDataOverrideInfo' structure contains the Absoccdata Override info.
    
    :var absoccDataType: Type of AbsoccData.
    :var additionalInfo: Additional Infomation about AbsoccData.
    """
    absoccDataType: int = 0
    additionalInfo: str = ''


@dataclass
class ChildLineInfo2(TcBaseObj):
    """
    'ChildLineInfo' contains child line object, index to 'DatasetInfo' and configuration accuracy flag.
    
    :var line: Line can be BOMLine or Fnd0BomLineLite. This object is added into the service data response, hence
    depending on the Business Object type, the respective properties will be loaded from the property policy.
    :var indexToAttachedObjInfos: A list of index to datasetInfos in 'ExpansionResponse'. 'DatasetInfos' at that index
    has information about Dataset, named references objects and relation name for the line.
    :var unSupportedInfo: 0 indicates that line has all supported data. This is default value.
    1 indicates that line has Modular Variants.
    2 indicates that line has Incremental Change.
    3 indicates that line has Modular Variants and Incremental Change.
    4 indicate that line has Classic Variants (pre 8.2 format) .
    5 indicates that line has Classic Variants (pre 8.2 format) and Modular Variants.
    6 indicates that line has Classic Variants (pre 8.2 format) and Incremental Change.
    7 indicates that line has Classic Variants (pre 8.2 format) and Modular Variants &amp; Incremental Change.
    """
    line: RuntimeBusinessObject = None
    indexToAttachedObjInfos: List[int] = ()
    unSupportedInfo: int = 0


@dataclass
class Controls2(TcBaseObj):
    """
    'Controls2' facilitates user to specify the payload and information to find related Dataset objects.
    
    :var expansionPref: Contains payload and Dataset retrieval preferences.
    Supported key, values and meaning are:
    
    Key: maxLevel
    Number of levels to be expanded for a expansion request. If not specified next immediate level is expanded. This is
    the default behavior.
    Other valid value is positive integer.
    
    Key: fetchRelatedObjects
    Specifies whether to return related Dataset objects as specified in relationCriteria. Valid values are 0 and 1.
    0 indicates do not return any related objects. This is the default value.
    1 indicates to return related objects as per relationCriteria.
    
    Key: minLinesWithRelatedObjects
    Specifies minimum lines to accumulate with related Dataset objects based on relationCriteria. This preference has
    significance only when fetchRelatedobjects is set 1.
    If not specified or specified as 0 related object check is not performed. Other valid value is positive integer.
    
    Key: outputPageSize
    Specifies number of child lines to return in response. When preference is not specified or passed as 0, all lines
    generated during operation are returned. Other valid value is any positive integer.
    
    Note that response is returned when either of maxLevel, minLinesWithRelatedObjects or outputPageSize criteria is
    satisfied.
    :var relationCriteria: 'RelationAndTypesCriteria' contains GRM relation name, criteria to retrieve named reference
    objects and 'DatasetTypeAndNamedRefs'. This information is used to retrieve related Dataset, named reference
    objects and FMS tickets.
    :var processLeafNodesFirst: Setting this control parameter will consider Leaf nodes first for processing till the
    level of expansion achieved in expansion operation.
    :var absoccFilterList: This lists the Absocc properties that should be considered (included) during Configuration.
    :var configurationSolveType: Setting this control parameter will evaluate following LWB properties during Expansion:
    &bull;   bl_variant_state
    &bull;   bl_is_occ_configured
    &bull;   bl_is_suppressed
    0 indicates above properties will not be evaluated during Expansion.
    1 indicates above properties will be evaluated during Expansion.
    Note: This parameter is not considered if the "Show Un Configured By Variants" and "Show UnConfigured By Occurrence
    Effectivity" on the BOMWindow are FALSE.
    """
    expansionPref: StringKeyToIntValue2 = None
    relationCriteria: List[RelationTypesAndCriteria] = ()
    processLeafNodesFirst: bool = False
    absoccFilterList: AbsoccDataIncludeList = None
    configurationSolveType: int = 0


@dataclass
class Controls3(TcBaseObj):
    """
    'Controls3' facilitates user to specify the payload and information to find related Dataset objects.
    
    :var expansionPref: Contains payload and Dataset retrieval preferences.
    
    Supported key, values and meaning are:
    
    Key: maxLevel
    Number of levels to be expanded for a expansion request. If not specified next immediate level is expanded. This is
    the default behavior.
    Other valid value is positive integer.
    
    Key: fetchRelatedObjects
    Specifies whether to return related Dataset objects as specified in relationCriteria. Valid values are 0 and 1.
    0 indicates do not return any related objects. This is the default value.
    1 indicates to return related objects as per relationCriteria.
    
    Key: outputPageSize
    Specifies number of child lines to return in response. When preference is not specified or passed as 0, all lines
    generated during operation are returned. Other valid value is any positive integer.
    
    Note that response is returned when either of maxLevel, minLinesWithRelatedObjects or outputPageSize criteria is
    satisfied.
    :var relationCriteria: 'RelationAndTypesCriteria' contains GRM relation name, criteria to retrieve named reference
    objects and 'DatasetTypeAndNamedRefs'. This information is used to retrieve related Dataset, named reference
    objects and FMS tickets.
    :var processLeafNodesFirst: Setting this control parameter will consider Leaf nodes first for processing till the
    level of expansion achieved in this operation.
    """
    expansionPref: StringKeyToIntValue2 = None
    relationCriteria: List[RelationTypesAndCriteria] = ()
    processLeafNodesFirst: bool = False


@dataclass
class DatasetInfo3(TcBaseObj):
    """
    'DatasetInfo' describes GRM relation name through which Dataset is attached with underlying object of
    Fnd0BOMLineLite or BOMLine. It also contains 'NamedRefInfo' for Dataset requested through 'DatasetTypeAndNamedRefs'.
    
    :var relationName: Name of GRM relation through which underlying object of Fnd0BOMLineLite or BOMLine is related to
    Dataset object.
    :var dataset: Dataset attached to Fnd0BOMLineLite or BOMLine.
    :var namedRefInfo: A list of named reference objects details for Dataset.
    """
    relationName: str = ''
    dataset: Dataset = None
    namedRefInfo: List[NamedRefInfo3] = ()


"""
Map with key as Fnd0BOMLineLite or BOMLine and 'ChildLineInfo' as values.
"""
ParentLineToChildLineInfo3 = Dict[RuntimeBusinessObject, List[ChildLineInfo2]]


"""
Map with key as Fnd0BOMLineLite or BOMLine and uids of the children lines as values.
"""
ParentLineToChildrenLineUidValues2 = Dict[RuntimeBusinessObject, List[str]]


"""
Map with string as key and integer as value.
"""
StringKeyToIntValue2 = Dict[str, int]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class IDSM1ClientInfo(TcBaseObj):
    """
    'IDSM1ClientInfo' structure holds the client information required to access the remote IDSM server.
    
    :var clientSiteId: Requesting client siteid.
    :var targetSiteId: Target siteid to which request has been sent.       
    
    :var clientNode: Node name of requesting client.
    :var userId: Logged in userid of requesting client
    :var userName: Logged in username of requesting client.
    :var osUserName: OS logged in username of requesting client node.        
    
    :var groupName: Group name of userid of requesting client.
    :var roleName: Role name of userid of requesting client.  
    """
    clientSiteId: int = 0
    targetSiteId: int = 0
    clientNode: str = ''
    userId: str = ''
    userName: str = ''
    osUserName: str = ''
    groupName: str = ''
    roleName: str = ''


@dataclass
class IDSM1EndAskInfoIn(TcBaseObj):
    """
    'IDSM1EndAskInfoIn' structure holds the information required to finish the info file transfer.
    
    :var clientInfo: Correlation id of the IDSM server.
    :var infoType: This is not used internally.
    :var infoStagingDir: Complete path of the staging directory.
    :var abortFlag: Flag to determine the aborted operation.
    """
    clientInfo: IDSM1ClientInfo = None
    infoType: int = 0
    infoStagingDir: str = ''
    abortFlag: bool = False


@dataclass
class IDSM1EndExportIn(TcBaseObj):
    """
    IDSM1EndExportIn structure holds the client information and the server staging directory required to complete the
    remote import operation.
    
    :var clientInfo: Object having the client information.
    :var exportStagingDir: Complete path of the server staging directory.
    :var abortFlag: Flag to determine, if the remote import operation needs to be aborted.
    """
    clientInfo: IDSM1ClientInfo = None
    exportStagingDir: str = ''
    abortFlag: bool = False


@dataclass
class IDSM1EndImportOut(TcBaseObj):
    """
    'IDSM11EndImportOut' structure holds the failures occurred during the remote export transaction.
    
    :var failures: List of failures on IDSM server, the list contain PUID of the failed object along with the ifail
    value.
    """
    failures: List[IdsmIeFailures] = ()


@dataclass
class IDSM1EndImportRes(TcBaseObj):
    """
    'IDSM11EndImportRes' structure holds the details required to access the remote IDSM server.
    
    :var out: List of IDSM failures.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM1EndImportOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM1LocateObjectIn(TcBaseObj):
    """
    'IDSM1LocateObjectIn' structure contains Integrated Distributed Services Manager (IDSM) client info and input
    object tag which needs to be located.
    
    
    :var clientInfo: Object having which has requesting client info.
    :var tagsAsString: Tag string of input object to be located.
    """
    clientInfo: IDSM1ClientInfo = None
    tagsAsString: str = ''


@dataclass
class IDSM1LocateObjectOut(TcBaseObj):
    """
    'IDSM1LocateObjectOut' structure holds owning site information and flag for object existence.
    
    :var found: Flag for object exists or not on remote site.
    :var owningSiteId: Siteid of Owning site of an object.
    :var idsmSiteId: SiteId of IDSM server.
    """
    found: bool = False
    owningSiteId: int = 0
    idsmSiteId: int = 0


@dataclass
class IDSM1LocateObjectRes(TcBaseObj):
    """
    'IDSM1LocateObjectRes' structure holds locating information for input object and 'ServiceData'.
    
    :var out: Object which has owning site info of input object.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM1LocateObjectOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM1RepublishObjectIn(TcBaseObj):
    """
    IDSM1RepublishObjectIn structure contains Integrated Distributed Services Manager (IDSM) client info and other
    information required to republish object to ODS.
    
    
    :var clientInfo: Object having which has requesting client info.
    :var tagsAsString: Tag string of input object to be republished.
    :var odsSiteId: Target ods siteid to which input object to be published.
    """
    clientInfo: IDSM1ClientInfo = None
    tagsAsString: str = ''
    odsSiteId: int = 0


@dataclass
class IDSM1StartAskInfoIn(TcBaseObj):
    """
    IDSM1StartAskInfoIn structure contains Integrated Distributed Services Manager (IDSM) client info and other
    information required to start getting schema info.
    
    
    :var clientInfo: Object which has requesting client info.
    :var infoType: Info type flag as IDSM_schema_info to get type-class information which is required to be fetched
    from remote site.
    """
    clientInfo: IDSM1ClientInfo = None
    infoType: int = 0


@dataclass
class IDSM1StartAskInfoOut(TcBaseObj):
    """
    'IDSM1StartAskInfoOut' structure provides staging directory information.
    
    :var infoFiles: List which has name of the file which has to be pulled by client during file transfer.
    
    :var infoStagingDir: Server side staging directory path, info files pushed to and pulled from this dir.
    """
    infoFiles: List[str] = ()
    infoStagingDir: str = ''


@dataclass
class IDSM1StartAskInfoRes(TcBaseObj):
    """
    'IDSM1StartAskInfoRes' structure provides information required to start importing schema files and ServiceData.
    
    
    :var out: Object which has staging dierctory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM1StartAskInfoOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM1VerifyObjectsIn(TcBaseObj):
    """
    'IDSM1VerifyObjectsIn' structure holds information required to verify input object and Integrated Distributed
    Services Manager (IDSM) client info.
    
    :var clientInfo: Object which has requesting client info.
    :var objectsToVerify: List of object tag string  to be verified in the database.
    """
    clientInfo: IDSM1ClientInfo = None
    objectsToVerify: List[str] = ()


@dataclass
class IDSM1VerifyObjectsOut(TcBaseObj):
    """
    'IDSM1VerifyObjectsOut' structure contains object verification status and failure code if any.
    
    :var verdicts: List of verdicts after verification of input objects 
                    Below are the valid values of verdict            
    -                  0 = does not exist
    -                  1 = exists
    -                  2 = exists as a readonly copy
    -                  3 = exists as a stub
    
    
    :var failureCodes: List of failure codes if any for each input object.
    """
    verdicts: List[int] = ()
    failureCodes: List[int] = ()


@dataclass
class IDSM1VerifyObjectsRes(TcBaseObj):
    """
    'IDSM1VerifyObjectsRes' structure contains object verification information retrieved from IDSM server and
    ServiceData.
    
    :var out: Object contains input objects verification status.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM1VerifyObjectsOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM1XferInfoFileIn(TcBaseObj):
    """
    'IDSM1XferInfoFileIn' structure holds the information required to transfer files to remote site and Integrated
    Distributed Services Manager (IDSM) client info.
    
    :var clientInfo: Object which has requesting client info.
    :var infoType: Info type flag as IDSM_schema_info to get type-class information which is required to be fetched
    from remote site.
    
    :var infoStagingDir: Server side staging directory path received in first call.
    :var infoFile: Info file which needs to be pulled in suring transfer.
    :var startingBlock: Transfer happens block by block, so index for starting block.
    """
    clientInfo: IDSM1ClientInfo = None
    infoType: int = 0
    infoStagingDir: str = ''
    infoFile: str = ''
    startingBlock: int = 0


@dataclass
class IDSM1XferInfoFileOut(TcBaseObj):
    """
    'IDSM1XferInfoFileOut' structure holds the file data information which is transferred to remote site.
    
    :var fileData: Actual file data which in the form of bytes.
    :var fileDataSize: File data size which is pulled from server.
    :var transferComplete: Flag to indicate whether all file transfer is complete or not.
    :var moreFiles: It transfers oen file at a time, so flag to check whether there are more files to be pulled in.
    """
    fileData: List[int] = ()
    fileDataSize: int = 0
    transferComplete: bool = False
    moreFiles: bool = False


@dataclass
class IDSM1XferInfoFileRes(TcBaseObj):
    """
    'IDSM1XferInfoFileRes' structure holds the file transfer output information.
    
    :var out: Object contains file transfer info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM1XferInfoFileOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM2DescribeObjectIn(TcBaseObj):
    """
    'IDSM2DescribeObjectIn' structure holds the information required to retrieve the business objects attribute
    value(s).
    
    :var clientInfo: An object having the client information.
    :var tagsAsString: Object tag unique for the Teamcenter server session.
    """
    clientInfo: IDSM1ClientInfo = None
    tagsAsString: str = ''


@dataclass
class IDSM2DescribeObjectOut(TcBaseObj):
    """
    'IDSM2DescribeObjectOut' structure holds the description of object hold at IDSM server.
    
    :var objectDesc: An object with the it s description, comprising  of site id, object tag, creataion date and last
    modified date as main constituent.
    """
    objectDesc: IdsmOldObjectDesc = None


@dataclass
class IDSM2DescribeObjectRes(TcBaseObj):
    """
    'IDSM2DescribeObjectRes' structure provides the object information retrieved from IDSM server.
    
    :var out: An object with the it s description, comprising  of site id, object tag, creataion date and last modified
    date as main constituent.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM2DescribeObjectOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM2VersionCheckIn(TcBaseObj):
    """
    'IDSM2VersionCheckIn' structure holds information required to get the IDSM server version.
    
    :var clientInfo: Object which has requesting client info.
    :var clientVersion: Requesting client version.
    """
    clientInfo: IDSM1ClientInfo = None
    clientVersion: int = 0


@dataclass
class IDSM2VersionCheckOut(TcBaseObj):
    """
    'IDSM2VersionCheckOut' structure holds the IDSM server version information.
    
    :var serverVersion: IDSM server version.
    :var clientRejected: Client rejected status if IDSM rejects client for some reason.
    """
    serverVersion: int = 0
    clientRejected: int = 0


@dataclass
class IDSM2VersionCheckRes(TcBaseObj):
    """
    'IDSM2VersionCheckRes' structure holds the IDSM version information and ServiceData.
    
    :var out: Object contains version info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM2VersionCheckOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM3EndImportIn(TcBaseObj):
    """
    'IDSM3EndImportIn' structure holds information required to complete the remote export transaction.
    
    :var clientInfo: Object having the client information.
    :var importObjects: PUIDs of the objects to be imported.
    :var excludeObjects: PUIDs of the objects to be excluded.
    :var importOptions: Import options.
    :var importStagingDir: Complete path of the import staging directory.
    :var abortFlag: Flag to determine the aborted operation.
    :var compressPerformedFlg: Compression flag to determine if compression of the meta file requires.
    """
    clientInfo: IDSM1ClientInfo = None
    importObjects: List[str] = ()
    excludeObjects: List[str] = ()
    importOptions: List[IdsmIeOpt] = ()
    importStagingDir: str = ''
    abortFlag: bool = False
    compressPerformedFlg: int = 0


@dataclass
class IDSM3ExportStatusIn(TcBaseObj):
    """
    'IDSM1ExportStatusIn' structure holds information required to retrieve the status of ongoing import/export
    transaction.
    
    :var clientInfo: Object having the client information.
    :var exportStagingDir: Complete path of the staging directory.
    :var abortExport: Abort export operation.
    :var checkClientAfterNWsos: After no. of workspace objects, check for client, used for polling trigger.
    """
    clientInfo: IDSM1ClientInfo = None
    exportStagingDir: str = ''
    abortExport: bool = False
    checkClientAfterNWsos: int = 0


@dataclass
class IDSM3ExportStatusOut(TcBaseObj):
    """
    'IDSM3ExportStatusOut' structure holds the import/export status returned by 'idsm3ExportStatusSvc' operation. The
    structure also provides information on failures occurred during the remote import/export transaction.
    
    :var exportComplete: Boolean to check, export completion status.
    :var workspaceObjects: Number of workspace objects processed.
    :var exportFiles: List of export files created for workspace object transfer.
    :var failures: List of IDSM import export failures.
    :var exportIfail: Export completion ifail code. It is set to abort ifail code if client ask for the abort.
    :var compressPerformedFlg: The compression flag set to true in order to perform the compression during export.
    """
    exportComplete: bool = False
    workspaceObjects: int = 0
    exportFiles: List[str] = ()
    failures: List[IdsmIeFailures] = ()
    exportIfail: int = 0
    compressPerformedFlg: int = 0


@dataclass
class IDSM3ExportStatusRes(TcBaseObj):
    """
    'IDSM3ExportStatusRes' structure holds the import/export status returned by 'idsm3ExportStatusSvc' operation.
    
    :var out: Export state of the object.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM3ExportStatusOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM3StartImportIn(TcBaseObj):
    """
    'IDSM3StartImportIn' structure contains Integrated Distributed Services Manager (IDSM) client info and required POM
    transmit file  Details, compression flag to start export.
    
    
    :var clientInfo: Object having which has requesting client info.
    :var xmitFname: Path of the multisite transmit file of exporting site copied to file store.
    
    :var compressFlg: Compression flag to check if Compression is active for input files.
    
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
     InfoZip .
    """
    clientInfo: IDSM1ClientInfo = None
    xmitFname: str = ''
    compressFlg: int = 0
    compressType: str = ''


@dataclass
class IDSM3StartImportOut(TcBaseObj):
    """
    'IDSM3StartImportOut' structure contains information about Multi-site staging directory and compression flag.
    
    
    :var importStagingDir: Server/importing side staging directory path in which exported objects files are  being
    transferred on the server.
    
    :var sendXmitFile: TRUE/FALSE, flag for whether to send POM transmit file from server.
    
    :var compressSupportedFlg: Compression flag to check if Compression is active on server/importing  site or not.
    """
    importStagingDir: str = ''
    sendXmitFile: bool = False
    compressSupportedFlg: int = 0


@dataclass
class IDSM3StartImportRes(TcBaseObj):
    """
    'IDSM3StartImportRes' structure provides staging directory information for starting export and  Servicedata.
    
    
    :var out: Object which has staging directory info.
    
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM3StartImportOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM4CheckSyncStatusIn(TcBaseObj):
    """
    'IDSM4CheckSyncStatusIn' structure holds the information required to retrieve the sync status of the business
    object(s).
    
    :var clientInfo: Object having the client information.
    :var commandToken: Type of operation needs to be performed. The below enum lists the supported operations.
    
    'enum' IDSM_sync_status_command_e
    {
        IDSM_CHECK_EXISTS,
        IDSM_CHECK_SYNCHRONIZABLE,
        IDSM_CHECK_IN_SYNC,
        IDSM_CHECK_IN_SYNC_GMT
    }
    
    :var objectsToCheck: List of PUIDs of the objects in consideration.
    :var lmdDates: Last modified dates of the respective objects.
    """
    clientInfo: IDSM1ClientInfo = None
    commandToken: int = 0
    objectsToCheck: List[str] = ()
    lmdDates: List[DateElement] = ()


@dataclass
class IDSM4CheckSyncStatusOut(TcBaseObj):
    """
    'IDSM4CheckSyncStatusOut' structure holds the business object sync status code and the failure code occurred during
    the sync operation.
    
    :var statusCodes: It is status of object w.r.t to IDSM server 
    - 0 == does not exist
    - 1 == exists
    - 2 == exists as a readonly copy
    - 3 == exists as a stub.
    
    
    :var failureCodes: List of ifail failure codes.
    """
    statusCodes: List[int] = ()
    failureCodes: List[int] = ()


@dataclass
class IDSM4CheckSyncStatusRes(TcBaseObj):
    """
    'IDSM4CheckSyncStatusRes' structure holds the information of the business object sync status with respect to the
    IDSM server.
    
    :var out: Sync state of the object, along with the failure codes.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM4CheckSyncStatusOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM4GetErrorStackIn(TcBaseObj):
    """
    IDSM4GetErrorStackIn structure holds information required to retrieve the error stack.
    
    :var clientInfo: Object having the client information.
    :var extraInt: Extra integer to convey information to the server.
    :var extraString: Extra string to convery information to the server.
    """
    clientInfo: IDSM1ClientInfo = None
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM4GetErrorStackOut(TcBaseObj):
    """
    'IDSM4GetErrorStackOut' structure holds the error stack.
    
    :var nStackEntries: Number of errors in the error stack.
    :var errorCodes: List of error codes.
    :var errorText: Error texts with respect to the error code.
    :var extraInt: Extra integer to convey the message to client. Now 1000 & 0 are used.
    :var extraString: Extra character(s) can be appended. E.g. The  \0 appended at the end.
    """
    nStackEntries: int = 0
    errorCodes: List[int] = ()
    errorText: List[str] = ()
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM4GetErrorStackRes(TcBaseObj):
    """
    'IDSM4GetErrorStackRes' structure holds the error stack along with the information required for logging.
    
    :var out: List of errors available in the stack.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM4GetErrorStackOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM4RemoteNotificationIn(TcBaseObj):
    """
    'IDSM4RemoteNotificationIn' structure contains Integrated Distributed Services Manager (IDSM)  client info and
    other  information required to notify remote site.
    
    
    :var clientInfo: Object which has requesting client info.
    :var eventTypeName: Event type string for sending notification type to the remote site. 
    
    :var replicatedObjects: List of tag strings of input objects for which the notiifiction required to send at remote
    site.
    """
    clientInfo: IDSM1ClientInfo = None
    eventTypeName: str = ''
    replicatedObjects: List[str] = ()


@dataclass
class IDSM4RemoteNotificationOut(TcBaseObj):
    """
    'IDSM4RemoteNotificationOut' structure provides failure code information.
    
    :var failureCodes: List of failure codes returned for each object given as an input.
    """
    failureCodes: List[int] = ()


@dataclass
class IDSM4RemoteNotificationRes(TcBaseObj):
    """
    'IDSM4RemoteNotificationRes' structure provides notification  information and 'ServiceData'.
    
    
    :var out: Object which has failure info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM4RemoteNotificationOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM4SetSyncOptionsIn(TcBaseObj):
    """
    'IDSM4SetSyncOptionsIn' structure contains Integrated Distributed Services Manager (IDSM) 
    client info and other information required to set sync options.
    
    
    :var clientInfo: Object which has requesting client info.
    :var optionsToken: Sync option token integer value. Below are the valid values of        
                  option token
    #define OBJIO_auto_sync                         1
    #define OBJIO_batch_sync                        2
    #define OBJIO_do_not_sync                       3
    #define OBJIO_auto_sync_and_notify            101
    #define OBJIO_batch_sync_but_notify           102
    #define OBJIO_do_not_sync_but_notify          103
    #define OBJIO_notify_and_sync_unchanged       104
    #define OBJIO_no_notify_and_sync_unchanged    105
    
    :var objectsOfInterest: List of object tag string for which sync options to be set.
    
    :var objectsNotifyOption: List of notification subscriptions option flag for each object in objectsOfInterest list.
    """
    clientInfo: IDSM1ClientInfo = None
    optionsToken: int = 0
    objectsOfInterest: List[str] = ()
    objectsNotifyOption: List[bool] = ()


@dataclass
class IDSM4SetSyncOptionsOut(TcBaseObj):
    """
    'IDSM4SetSyncOptionsOut' structure provides failure code info per object.
    
    :var failureCodes: List for failure code for each input object.
    """
    failureCodes: List[int] = ()


@dataclass
class IDSM4SetSyncOptionsRes(TcBaseObj):
    """
    'IDSM4SetSyncOptionsRes' structure provides information returned after setting sync options
     and 'ServiceData'.
    
    
    :var out: Object which has failure info if any.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM4SetSyncOptionsOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM4XferExportFileIn(TcBaseObj):
    """
    'IDSM4XferExportFileIn' structure holds the client and server information required to complete the file transfer
    from target site to source site.
    
    :var clientInfo: Object having the client information.
    :var exportStagingDir: Complete path of the server side staging directory.
    :var exportFile: Export file name.
    :var startingPosition: This attribute not used, pass 0.
    :var command: For import case pass CMD_get.    
    'enum' FTCommand 
    { 
        CMD_send, 
        CMD_get, 
        CMD_abort, 
        CMD_close 
    }
    
    :var extraInt: This attribute not used, pass 0.
    :var extraString: This attribute not used pass empty string.
    """
    clientInfo: IDSM1ClientInfo = None
    exportStagingDir: str = ''
    exportFile: str = ''
    startingPosition: int = 0
    command: int = 0
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM4XferExportFileOut(TcBaseObj):
    """
    'IDSM4XferExportFileOut' structure holds the information of remote server and the transferred file.
    
    :var portNumber: IDSM Server port number.
    :var serverId: IDSM server id.
    :var fileSize: File size of the transferred file.
    :var extraInt: This attribute not used, and returned as zero.
    :var extraString: This attribute not used and returned as empty string.
    """
    portNumber: int = 0
    serverId: int = 0
    fileSize: str = ''
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM4XferExportFileRes(TcBaseObj):
    """
    'IDSM4XferExportFileRes' structure holds the remote server information, returned by the 
    'idsm4XferExportFileSvc' operation.
    
    
    :var out: Object of IDSM4XferExportFileOut structure.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM4XferExportFileOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM4XferImportFileIn(TcBaseObj):
    """
    'IDSM4XferImportFileIn' structure holds the client and server information required to complete the file transfer
    from source site to target site.
    
    :var clientInfo: Object having the client information.
    :var importStagingDir: Complete path of the server side staging directory.
    :var importFile: Import file name.
    :var fileSize: File size.
    :var command: For import case pass CMD_send.
    'enum' FTCommand 
    { 
        CMD_send, 
        CMD_get, 
        CMD_abort, 
        CMD_close 
    }
    
    :var extraInt: This attribute not used, pass 0.
    :var extraString: This attribute not used pass empty string.
    """
    clientInfo: IDSM1ClientInfo = None
    importStagingDir: str = ''
    importFile: str = ''
    fileSize: str = ''
    command: int = 0
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM4XferImportFileOut(TcBaseObj):
    """
    'IDSM4XferImportFileOut' structure holds the information of remote server and the transferred file.
    
    :var portNumber: IDSM Server port number.
    :var serverId: IDSM server id.
    :var extraInt: This attribute not used, and returned as zero.
    :var extraString: This attribute not used, and returned as zero.
    """
    portNumber: int = 0
    serverId: int = 0
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM4XferImportFileRes(TcBaseObj):
    """
    'IDSM4XferImportFileRes' structure holds the remote server information, returned by the 
    'idsm4XferImportFileSvc' operation.
    
    
    :var out: Object of 'IDSM4XferExportFileOut' structure.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM4XferImportFileOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM6DescribeItemIn(TcBaseObj):
    """
    'IDSM6DescribeItemIn' structure holds the information required to retrieve the item s attribute value(s).
    
    :var clientInfo: Object having the client information.
    :var itemId: Unique item id.
    """
    clientInfo: IDSM1ClientInfo = None
    itemId: str = ''


@dataclass
class IDSM6DescribeItemOut(TcBaseObj):
    """
    'IDSM6DescribeItemOut' structure holds the values and names of attributes retrieved from the IDSM server.
    
    :var attributes: List provides values for an item attribute name.
    :var attributeNames: List provides the available attribute names for an item.
    """
    attributes: List[str] = ()
    attributeNames: List[str] = ()


@dataclass
class IDSM6DescribeItemRes(TcBaseObj):
    """
    'IDSM6DescribeItemRes' structure provides the item information retrieved from IDSM server.
    
    :var out: Attributes values of item.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM6DescribeItemOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM6EndExportItemIdListIn(TcBaseObj):
    """
    'IDSM6EndExportItemIdListIn' structure holds the necessary information to finish the remote item id import
    operation.
    
    :var clientInfo: Object having the client information.
    :var exportStagingDir: Complete path of the server side staging directory.
    """
    clientInfo: IDSM1ClientInfo = None
    exportStagingDir: str = ''


@dataclass
class IDSM6EndSendObjectsIn(TcBaseObj):
    """
    'IDSM6EndSendObjectsIn' structure holds the client information required to execute the distributed_execute utility
    remotely.
    
    :var clientInfo: Object having the client information.
    :var serverStagingDir: Complete path of the server staging directory.
    :var abortFlag: Flag to determine, if the export operation to be aborted after object send complete.
    """
    clientInfo: IDSM1ClientInfo = None
    serverStagingDir: str = ''
    abortFlag: bool = False


@dataclass
class IDSM6PerformSendObjectsIn(TcBaseObj):
    """
    'IDSM6PerformSendObjectsIn' structure contains Integrated Distributed Services Manager (IDSM) client info and other
    information required to send system objects to remote site.
    
    
    :var clientInfo: Object having which has requesting client info.
    :var serverStagingDir: Staging directory, files pushed from and pulled to this dir.
    :var emailAddresses: List of email addresses. Array of email addresses to which to send remote report.( Number of
    email addresses == 0  implies no need to generate remote report file, unless required by report_file parameter or
    remote email reference.)
    
    :var substitutions: List of 'SendObjectsSubstitutions' objects which has attribute substitution specifications.
    
    :var retentions: List of 'SendObjectsRetentions' objects which has attribute retentions.
    
    :var compressPerformedFlg: Compression flag to check if Compression is active.
    :var commandToPerform: Type of operation need to perform. The below enum 
                   lists the supported operations.
    typedef enum IDSM_send_sys_objects_cmd_e
    {
        IDSM_DSA_distribute_op,        
        IDSM_DSA_preview_distribute_op, 
        IDSM_DSA_diff_op                
    } IDSM_send_sys_objects_cmd_t;
    #define DSAM_item_report_merge          665
    #define DSAM_item_report_item_traversal 666
    #define DSAM_item_report                667
    #define DSAM_cleanup_shared_objects     668
    #define DSAM_fix_release_status         669
    #define DSAM_purge_datasets             670
    #define DSAM_export_recovery            671
    #define DSAM_item_export                672
    #define DSAM_datashare                  673
    
    :var commandOptionFlags: Operation command option flag.
    :var abortOnError: Flag to deteremine aborting on error.
    :var objectsXmlFile: XML objects file to send to remote site.
    :var plmxmlFileFormat: TRUE/FALSE, XML objects file is in plmxml format.
    :var objectsXmlFileOut: If sending objects for comparison or diff, name of XML file with remote objects to be
    pulled back.
    
    :var errorFile: Name of remote error file to pull back. (NULL implies no need to generate error file.).
    
    :var reportFile: Name of remote report file to pull back.(NULL implies no need to  generate remote report file,
    unless required by email_addresses  or remote email preference.).
    """
    clientInfo: IDSM1ClientInfo = None
    serverStagingDir: str = ''
    emailAddresses: List[str] = ()
    substitutions: List[SendObjectsSubstitutions] = ()
    retentions: List[SendObjectsRetentions] = ()
    compressPerformedFlg: int = 0
    commandToPerform: int = 0
    commandOptionFlags: int = 0
    abortOnError: bool = False
    objectsXmlFile: str = ''
    plmxmlFileFormat: bool = False
    objectsXmlFileOut: str = ''
    errorFile: str = ''
    reportFile: str = ''


@dataclass
class IDSM6PerformSendObjectsOut(TcBaseObj):
    """
    'IDSM6PerformSendObjectsOut' structure contains export/import status information.
    
    :var returnFiles: List of output files to be retruned to the client. It contains report file, status file etc for
    export/import operation.
    
    :var compressSupportedFlg: Compression flag to check if Compression is active.
    """
    returnFiles: List[str] = ()
    compressSupportedFlg: int = 0


@dataclass
class IDSM6PerformSendObjectsRes(TcBaseObj):
    """
    'IDSM6PerformSendObjectsRes' structure provides information returned after sending objects to remote site and
    ServiceData.
    
    
    :var out: Object which has status info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM6PerformSendObjectsOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM6StartExportItemIdListIn(TcBaseObj):
    """
    'IDSM6StartExportItemIdListIn' structure contains Integrated Distributed Services Manager (IDSM) client info and
    other information required to start export itemids list.
    
    
    :var clientInfo: Object which has requesting client info.
    :var queryStrings: List of query strings for querying itemids.
    :var compressFlg: Compression flag to check if Compression is active.
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
      InfoZip .
    """
    clientInfo: IDSM1ClientInfo = None
    queryStrings: List[str] = ()
    compressFlg: int = 0
    compressType: str = ''


@dataclass
class IDSM6StartExportItemIdListOut(TcBaseObj):
    """
    'IDSM6StartExportItemIdListOut' structure provides staging directory and other information.
    
    
    :var exportFiles: List  which has name of the file which has to be pulled by client during file transfer.
    
    :var exportStagingDir: Server side staging directory path, exportFiles files pushed to and pulled from this dir.
    
    :var compressPerformedFlg: Compression flag to check whether exportFiles are compressed or not. IF compression is
    done set the correct flag for decompressing later in the flow.
    """
    exportFiles: List[str] = ()
    exportStagingDir: str = ''
    compressPerformedFlg: int = 0


@dataclass
class IDSM6StartExportItemIdListRes(TcBaseObj):
    """
    'IDSM6StartExportItemIdListRes' structure provides information required to start import and ServiceData.
    
    
    :var out: Object which has staging directory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM6StartExportItemIdListOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM6StartSendObjectsIn(TcBaseObj):
    """
    'IDSM6StartSendObjectsIn' Integrated Distributed Services Manager (IDSM)  client information and compression
    details.
    
    
    :var clientInfo: Object which has requesting client info.
    :var compressFlg: Compression flag to check if Compression is active or not.
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
      InfoZip .
    """
    clientInfo: IDSM1ClientInfo = None
    compressFlg: int = 0
    compressType: str = ''


@dataclass
class IDSM6StartSendObjectsOut(TcBaseObj):
    """
    'IDSM6StartSendObjectsOut' structure contains server side staging directory and compression flag.
    
    :var serverStagingDir: Server/importing side staging directory path in which object XML files are  being
    transferred on the server.
    
    :var compressSupportedFlg: Compression flag to check if Compression is active on server/importing  site or not.
    """
    serverStagingDir: str = ''
    compressSupportedFlg: int = 0


@dataclass
class IDSM6StartSendObjectsRes(TcBaseObj):
    """
    'IDSM6StartSendObjectsRes' structure contains information required to start the export and ServiceData.
    
    :var out: Object which has staging directory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM6StartSendObjectsOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM7AskVersionIn(TcBaseObj):
    """
    'IDSM7AskVersionIn' structure holds the Integrated Distributed Services Manager (IDSM) client information required
    to access the remote IDSM server.
    
    :var clientInfo: Object which has requesting client info.
    """
    clientInfo: IDSM1ClientInfo = None


@dataclass
class IDSM7AskVersionOut(TcBaseObj):
    """
    'IDSM7AskVersionOut' structure holds IDSM server major and minor version.
    
    :var serverMajorVersion: IDSM server major version.
    :var serverMinorVersion: IDSM server minor version.
    :var serverQrmNumber: IDSM server qrm number.
    :var serverIrmNumber: IDSM server irm number.
    """
    serverMajorVersion: int = 0
    serverMinorVersion: int = 0
    serverQrmNumber: int = 0
    serverIrmNumber: int = 0


@dataclass
class IDSM7AskVersionRes(TcBaseObj):
    """
    'IDSM7AskVersionRes' structure holds IDSM server version information and ServiceData.
    
    :var out: Object which has version info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM7AskVersionOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM7DistributedAppIn(TcBaseObj):
    """
    'IDSM7DistributedAppIn' structure holds the client information required to make the distributed call.
    
    :var clientInfo: An object having the client information.
    :var imanVersion: Teamcenter client version, internally this is converted to the client major version and minor
    version.
    :var imanIrmNumber: Teamcenter patch number.
    :var appName: Name of the application, client want to check service for.
    :var appOpCode: Op code of the service.
    - DIST_IDSM_commit_synchronous_remote_import_op          0
    - DIST_IDSM_exchange_supported_feature_set_op            1
    - DIST_IDSM_ask_imported_sst_uids_op                     2
    - DIST_IDSM_perform_checkpoint_function_op               3
    - DIST_IDSM_identify_sst_client_op                       4
    - DIST_IDSM_is_process_dead_op                           5
    - DIST_IDSM_ask_server_process_info_op                   6
    - DIST_IDSM_describe_object_l10n_attr_op                 7
    - DIST_IDSM_start_plmxml_sync_op                         8
    - DIST_IDSM_set_remote_site_logger_level_op           9
    
    
    :var appInputStrings: List of input strings for the controller.
    """
    clientInfo: IDSM1ClientInfo = None
    imanVersion: int = 0
    imanIrmNumber: int = 0
    appName: str = ''
    appOpCode: int = 0
    appInputStrings: List[str] = ()


@dataclass
class IDSM7DistributedAppOut(TcBaseObj):
    """
    'IDSM7DistributedAppOut' structure holds the strings returned by the controller.
    
    :var appOutputStrings: List of strings returned by the controller.
    """
    appOutputStrings: List[str] = ()


@dataclass
class IDSM7DistributedAppRes(TcBaseObj):
    """
    'IDSM7DistributedAppRes' structure holds the description of the service.
    
    :var out: An object with the service description.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM7DistributedAppOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM7StartExportIn(TcBaseObj):
    """
    'IDSM7StartExportIn' structure contains Integrated Distributed Services Manager (IDSM) client info and other
    information required for export.
    
    
    :var clientInfo: Object which has requesting client info.
    :var exportObjects: List of tag string of objects to be exported.
    :var excludeObjects: List of tag string of objects to be excluded during export.
    
    :var includeObjects: String vector of tag string of objects to be included during export.
    
    :var reason: User input to explain, why the remote import of business object required. Its not a mandatory input
    and user can pass empty string.
    
    :var exportOptions: List of objects which has IDSM Import and Export options.
    :var compressFlg: Compression flag to check if Compression is active.
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
      InfoZip .
    
    :var checkClientAfterNWsos: After no. of workspace objects, check for client, used for polling trigger.
    """
    clientInfo: IDSM1ClientInfo = None
    exportObjects: List[str] = ()
    excludeObjects: List[str] = ()
    includeObjects: List[str] = ()
    reason: str = ''
    exportOptions: List[IdsmIeOpt] = ()
    compressFlg: int = 0
    compressType: str = ''
    checkClientAfterNWsos: int = 0


@dataclass
class IDSM7StartExportOut(TcBaseObj):
    """
    'IDSM7StartExportOut' structure contains information about Multi-site staging directory and compression flag.
    
    
    :var exportFiles: Listof export files which has exported objects info created in export staging directory.
    
    :var exportStagingDir: Export staging directory path in which exportFiles files are pushed to and read from this
    directory.
    
    :var failures: Object  which has IDSM failure information.
    :var compressPerformedFlg: Compression flag to check whether exportFiles are compressed or not. IF compression is
    done set the correct flag for decompressing later in the flow.
    """
    exportFiles: List[str] = ()
    exportStagingDir: str = ''
    failures: List[IdsmIeFailures] = ()
    compressPerformedFlg: int = 0


@dataclass
class IDSM7StartExportRes(TcBaseObj):
    """
    'IDSM7StartExportRes' structure provides staging directory information for starting import and ServiceData.
    
    
    :var out: Object which has staging directory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM7StartExportOut = None
    serviceData: ServiceData = None


@dataclass
class IDSM9DescribeObjectIn(TcBaseObj):
    """
    'IDSM9DescribeObjectIn' structure holds the information required to retrieve the business object s attribute
    value(s).
    
    :var clientInfo: An object having the client information.
    :var tagsAsString: Object tag unique for the Teamcenter server session.
    """
    clientInfo: IDSM1ClientInfo = None
    tagsAsString: str = ''


@dataclass
class IDSM9DescribeObjectOut(TcBaseObj):
    """
    'IDSM9DescribeObjectOut' structure holds the description of object hold at IDSM server.
    
    :var objectDesc: The attribute provides the complete information of the object.
    """
    objectDesc: IdsmObjectDesc = None


@dataclass
class IDSM9DescribeObjectRes(TcBaseObj):
    """
    IDSM9DescribeObjectRes structure provides the object information retrieved from IDSM server.
    
    :var out: An object with the it s description, comprising  of site id, object tag, creataion date and last modified
    date as main constituent.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    out: IDSM9DescribeObjectOut = None
    serviceData: ServiceData = None


@dataclass
class IdsmIeFailures(TcBaseObj):
    """
    'IdsmIeFailures' structure holds the list of object failed to remote export & the respective failures occurred.
    
    :var failingObject: PUID of the failing object.
    :var failureCode: ifail value.
    """
    failingObject: str = ''
    failureCode: int = 0


@dataclass
class IdsmIeOpt(TcBaseObj):
    """
    'IdsmIeOpt' structure holds information which will influence the remote export transaction.
    
    :var optType: An import option.
    :var optValue: Value of an import option.
    """
    optType: int = 0
    optValue: IdsmIeOptVal = None


@dataclass
class IdsmIeOptVal(TcBaseObj):
    """
    'IdsmIeOptVal' structure holds information which will influence the remote export transaction.
    
    :var optVal: Value of an import option.
    :var stringOptVal: Name of the an import option.
    """
    optVal: int = 0
    stringOptVal: str = ''


@dataclass
class IdsmObjectDesc(TcBaseObj):
    """
    IdsmObjectDesc structure holds description of the object hold at IDSM server.
    
    :var siteId: Unique id of site where object belongs to.
    :var tagAsString: Tag of the object unique within Teamcenter session.
    :var objectLastModDate: Last modified date of the object.
    :var objectCreationDate: Creation date of the object.
    :var objectReleaseStatusNames: Release status name applied to the object.
    :var customAttrNames: Any custom attributes applied on the object.
    :var customAttrValues: Values of the custom attributes.
    :var objectId: Unique id of the object.
    :var objectRevId: Revision id of the object.
    :var objectName: Name of the object.
    :var objectDesc: Description of the object.
    :var objectClass: Class of the object.
    :var objectType: Type of the object.
    :var objectOwner: Owner who has object ownership.
    :var objectGroup: Group who has access to this object.
    """
    siteId: int = 0
    tagAsString: str = ''
    objectLastModDate: datetime = None
    objectCreationDate: datetime = None
    objectReleaseStatusNames: List[str] = ()
    customAttrNames: List[str] = ()
    customAttrValues: List[str] = ()
    objectId: str = ''
    objectRevId: str = ''
    objectName: str = ''
    objectDesc: str = ''
    objectClass: str = ''
    objectType: str = ''
    objectOwner: str = ''
    objectGroup: str = ''


@dataclass
class IdsmOldObjectDesc(TcBaseObj):
    """
    IdsmOldObjectDesc structure holds description of the object hold at IDSM server.
    
    
    :var siteId: Unique id of site where object belongs to.
    :var tagAsString: Tag of the object unique within Teamcenter session.
    :var objectCreationDate: Creation date of the object.
    :var objectReleaseStatusNames: Release status name applied to the object.
    :var objectId: Unique id of the object.
    :var objectName: Name of the object.
    :var objectDesc: Description of the object.
    :var objectClass: Class of the object.
    :var objectType: Type of the object.
    :var objectOwner: Owner who has object ownership.
    :var objectGroup: Group who has access to this object.
    :var objectLastModDate: Last modified date of the object.
    """
    siteId: int = 0
    tagAsString: str = ''
    objectCreationDate: datetime = None
    objectReleaseStatusNames: List[str] = ()
    objectId: str = ''
    objectName: str = ''
    objectDesc: str = ''
    objectClass: str = ''
    objectType: str = ''
    objectOwner: str = ''
    objectGroup: str = ''
    objectLastModDate: datetime = None


@dataclass
class SendObjectsRetentions(TcBaseObj):
    """
    'SendObjectsRetentions' structure contains object retention info.
    
    :var className: Class name of the object.
    :var attrName: Attribute name of the object.
    """
    className: str = ''
    attrName: str = ''


@dataclass
class SendObjectsSubstitutions(TcBaseObj):
    """
    'SendObjectsSubstitutions' structure contains object substitution info.
    
    :var className: Class name of the object.
    :var attrName: Attribute name of the object.
    :var attrValue: Attribute value for a given attribute name.
    :var substitutionValue: Substitution value of an attribute.
    """
    className: str = ''
    attrName: str = ''
    attrValue: str = ''
    substitutionValue: str = ''


@dataclass
class DateElement(TcBaseObj):
    """
    DateElement structure holds the timestamp of the business object, and used to capture its last modified date.
    
    :var dateVal: Holds last modified dates of the respective object.
    """
    dateVal: datetime = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, VariantRule, Awb0BOMIndexAdminData, RevisionRule, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ProcessBomIndexInfo(TcBaseObj):
    """
    This structure contains  the processing options and the instances of Awb0BOMIndexAdminData whose state are either
    'ReadyToIndex', 'IndexGenSuccess', 'IndexExportSuccess', 'SolrIndexGenSuccess' and 'MarkedForDeletion'.
    
    :var biadObjs: Awb0BOMIndexAdminData  instances which has to be  processed for indexing.
    :var processingOptions: A map that contains the option and values. 
    The following options are currently supported
    Key:'updateTIEAMTables'
    Value:'true'
    Description: Update the TIE Access Manager(AM) tables if there is a change in the AM rule tree.
    """
    biadObjs: List[Awb0BOMIndexAdminData] = ()
    processingOptions: StringToStringMap = None


@dataclass
class ProcessBomIndexOutput(TcBaseObj):
    """
    This structure contains the processed Awb0BOMIndexAdminData instance,  FMS file tickets for the 'TCXML' and export
    log file. If the indexes were up-to-date then 'tcxmlFileTicket' would be an empty string. If  the 
    Awb0BOMIndexAdminData instance was marked for deletion then the 'tcxmlFileTicket' and the 'exportLogFileTicket'
    would be empty.
    
    :var biadObj: Awb0BOMIndexAdminData  instance which was processed.
    :var tcxmlFileTicket: Transient file ticket of the tcxml file.
    :var exportLogFileTicket: Transient file ticket of the export log  file.
    """
    biadObj: Awb0BOMIndexAdminData = None
    tcxmlFileTicket: str = ''
    exportLogFileTicket: str = ''


@dataclass
class ProcessBomIndexResponse(TcBaseObj):
    """
    This structure contains a list of the processed Awb0BOMIndexAdminData  instances,  the transient file ticket to a
    file that contains the 'TCXML' representation of configured product indexes and the transient file ticket to export
    log file that was created during index generation.
    The 'outputList' is ordered in same order as input 'bomIndexObjs'.
    
    :var outputList: A list of 'ProcessBomIndexOutput' structures.
    :var serviceData: The Service Data through which the errors are communicated to the client.
    Following are some possible errors returned in ServiceData:
    - 203408 - Transfer Option Set could not be found.
    - 11001 -  TIE internal error.
    - 11112 - No configured revision is found for the given top line Item.
    
    """
    outputList: List[ProcessBomIndexOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreBOMIndxAdmDataInfo(TcBaseObj):
    """
    'CreBOMIndxAdmDataInfo' contains the product and configuration rules information for which the 'BOM' index should
    be generated.
    
    :var product: The product can be ItemRevision or CollaborativeDesign, for which  the 'BOM' index should be
    generated.
    :var cfgRuleInfos: List of configuration rules including RevisionRule(s) and SavedVariantRule(s) to use for
    configuring the given product.
    """
    product: BusinessObject = None
    cfgRuleInfos: List[IndxCnfgRulesInfo] = ()


@dataclass
class CreBOMIndxAdmDataResp(TcBaseObj):
    """
    'CreBOMIndxAdmDataResp' contains the list of Awb0BOMIndexAdminData instances that were created or updated.
    
    :var bomIndexAdminDataObjs: List of Awb0BOMIndexAdminData instances that are created or updated.
    :var serviceData: The 'ServiceData' through which the errors are communicated to the client.
    """
    bomIndexAdminDataObjs: List[Awb0BOMIndexAdminData] = ()
    serviceData: ServiceData = None


@dataclass
class IndxCnfgRulesInfo(TcBaseObj):
    """
    'IndxCnfgRulesInfo' contains the list of RevisionRule(s) with the effectivity information and a list of saved
    VariantRule(s) with their owning ItemRevision(s) which are applicable to the given product. The configuration rules
    will be used to configure the structure for indexing. For each of saved VariantRule the owning ItemRevision must
    also be specified.
    
    :var revisionRuleInfo: The base RevisionRule used to configure the product.
    :var variantRules: List of saved VariantRule(s) used for structure configuration.
    :var svrOwningItemRevs: List of ItemRevision(s) to which the saved VariantRule(s) are associated.
    """
    revisionRuleInfo: IndxRevisionRuleInfo = None
    variantRules: List[VariantRule] = ()
    svrOwningItemRevs: List[ItemRevision] = ()


@dataclass
class IndxRevisionRuleInfo(TcBaseObj):
    """
    Structure containing the revision rules with effectivity info used to configure a product in the BOM Index
    
    :var revisionRule: The base RevisionRule used to configure the product.
    :var effectivityEndItem: The end item used as RevisionRule effectivity criteria; it can be an Item or ItemRevision.
    :var effectivityDate: The effectivity date used as RevisionRule effectivity criteria.
    :var effectivityUnit: The effectivity unit number used as RevisionRule effectivity criteria along with effectivity
    end item. The unit number is a positive interger. Initialize it with -1 if unit number is not to be considered.
    """
    revisionRule: RevisionRule = None
    effectivityEndItem: BusinessObject = None
    effectivityDate: datetime = None
    effectivityUnit: int = 0


"""
A map to associate a string key to a string value.
"""
StringToStringMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Internal.IcsAw._2019_12.Classification import FindClassificationInfo3Response
from typing import List
from tcsoa.gen.Internal.IcsAw._2018_05.Classification import SearchCriteria
from tcsoa.base import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class ClassificationService(TcService):

    @classmethod
    def findClassificationInfo3(cls, workspaceObjects: List[WorkspaceObject], searchCriterias: List[SearchCriteria], classificationDataOptions: int) -> FindClassificationInfo3Response:
        """
        Finds classification information based on the input criteria.
        
        A populated list If a list of WorkspaceObject objects are provided ; then this operation finds all the
        classification objects for the given WorkspaceObject.
        In caseIf a given WorkspaceObject is not classified this operation finds and returns standalone classification
        object; which could then be linked to given WorkspaceObject.
        If a given workspace object is not classified and also the system does not contain standalone classification
        object which could be linked to the WorkspaceObject then this operation will return classification hierarchy
        which could be used for browsing and classifying an object in a classification class of interest. (For
        traditional classification system, children of "Classification Root" (ICM) are returned. For next generation
        Classification Standard Taxonomy (CST) system, children of the Default Hierarchy are returned).
        Empty list of WorkspaceObject would be passed in if user wants to search for a class in classification
        hierarchy using search criteria.
        This operations takes care of returning classification node information from next generation classification
        system including, containing cardinal and polymorphic blocks.
        
        If a search criteria is provided; then this operation searches for classification classes in classification
        hierarchy and returns their metadata information like class ID, name, class parents etc. If both
        searchCriterias and workspaceObjects are provided, workspaceObjects list is ignored and search criteria is used.
        
        Use cases:
        This operation could be used when user needs to find classification objects, for both traditional and
        Classification Standard Taxonomy (CST), based on workspace objects. Each time a WorkspaceObject is classified
        in a classification class a classification object is created. After searching for all the classification
        objects corresponding to a WorkspaceObject, user can find more information about the classification(s) of a
        WorkspaceObject. 
        The operation findClassificationInfo2 can be used to get detailed information about the classification objects,
        classification classes where these classification objects belong. After getting information about
        classification objects the user can choose to modify or delete the objects by using operation:
        saveClassificationObjects2 or deleteClassificationObjects, both of which support traditional and Classification
        Standard Taxonomy (CST) classes.
        
        The operation findClassificationInfo2 can also be used when a user wants to search for classes in
        classification hierarchy by using any of class' properties. After searching for classification classes; the
        user can use that information to browse through a subset of classification hierarchy and classify a
        classifiable workspace object in a classification class of interest.
        """
        return cls.execute_soa_method(
            method_name='findClassificationInfo3',
            library='Internal-IcsAw',
            service_date='2019_12',
            service_name='Classification',
            params={'workspaceObjects': workspaceObjects, 'searchCriterias': searchCriterias, 'classificationDataOptions': classificationDataOptions},
            response_cls=FindClassificationInfo3Response,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SelectedStudySourceResponse(TcBaseObj):
    """
    Returns the FMS file ticket to the log and service data with partial errors if any.
    Following partial errors may be returned.
    251087    The selected objects are not supported by "Synchronize" operation.
    251088    The mode "abc" is not a valid synchronization mode.
    
    :var serviceData: The service data containing partial errors if any.
    :var logFileTicket: The FMS ticket to the log file.
    """
    serviceData: ServiceData = None
    logFileTicket: str = ''


@dataclass
class SelectedSyncPublishStudyInput(TcBaseObj):
    """
    A list of selected business objects from Simulation Study. The valid types are Mfg0BvrSimStudy, Mfg0BvrProcessArea,
    Mfg0BvrProcessLine, Mfg0BvrProcessStation Mfg0BvrProcess, and Mfg0BvrOperation.It also contains the study root node
    and the synchronization mode (Time based/ Event based).
    
    :var selectedLines: A list of selected objects in a simulation study.The valid types are: Mfg0BvrSimStudy,
    Mfg0BvrProcessArea, Mfg0BvrProcessLine, Mfg0BvrProcessStation, Mfg0BvrProcess, Mfg0BvrOperation and Mfg0BvrWorkarea.
    :var mode: The  mode to synchronize the study. The valid values are: "TimeBased", which performs a full
    synchronization and "EventBased", which performs a partial synchronization.
    """
    selectedLines: List[BusinessObject] = ()
    mode: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, ScheduleTask, Schedule, ResourceAssignment, TaskDependency
from tcsoa.gen.Internal.ProjectManagement._2008_06.ScheduleManagement import GenericAttributesContainer
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class PasteTaskContainer(TcBaseObj):
    """
    This Container will hold information required for cut paste or copy paste of Task object.
     Teamcenter::WorkspaceObject sourceTask --> source task
     Teamcenter::WorkspaceObject prevSibling --> destination task under which copied or cut task will be pasted.
     Teamcenter::WorkspaceObject newParent --> parent task
     int flag ---> 0= copy operation 1= cut operation
    
    
    
    :var sourceTask: The  task to copy or cut.
    :var prevSibling: The task above the requested location.
    :var newParent: The new parent of the Task.
    :var flag: flag =0 copy operation
    flag =1 cut operation
    """
    sourceTask: WorkspaceObject = None
    prevSibling: WorkspaceObject = None
    newParent: WorkspaceObject = None
    flag: int = 0


@dataclass
class DeferredSaveOption(TcBaseObj):
    """
    This will have information about schedule and deferred options as below
    -1 = Start deferred session and continue.
    0 = Save deferred session and continue.
    1= Save deferred session and exit.
    2= Cancel deferred session and exit.
    
    :var schedule: schedule tag which is in deferred mode.
    :var deferredOption: -1 = start deferred session and continue.
    0 = Save deferred session and continue.
    1= Save deferred session and exit.
    2= Cancel deferred session and exit.
    """
    schedule: Schedule = None
    deferredOption: int = 0


@dataclass
class DeferredSaveResponse(TcBaseObj):
    """
    Deferred save response  will have the information of updated a schedule.  It contains new, updated, and deleted
    tasks, resource assignments, and task dependencies information for a schedule.
    
    :var schedule: Schedule tag which is in deferred mode.
    :var scheduleUpdates: the schedule updates- GenericAttributesContainer{ object-schedule object
    tag,keyValContainer(can be empty),StringValContainer(can be empty),typedAttributeContainer(can be empty) }
    :var deletedAssignments: The deleted assignments
    :var newProxyTasks: The information of new ProxyTask created in this schedule
    :var proxyTaskUpdates: A list of known updated proxy tasks.
    :var deletedProxyTasks: A list of proxy tasks to be deleted.
    :var serviceData: The service data.
    :var newTasks: The new tasks to add
    :var newDependencies: The new dependencies
    :var newAssignments: The new assignments
    :var taskUpdates: The existing tasks update
    :var dependencyUpdates: The existing dependencies updates
    :var assignmentUpdates: The existing assignments update
    :var deletedTasks: The deleted tasks
    :var deletedDependencies: The deleted dependencies
    """
    schedule: Schedule = None
    scheduleUpdates: GenericAttributesContainer = None
    deletedAssignments: List[ResourceAssignment] = ()
    newProxyTasks: List[GenericAttributesContainer] = ()
    proxyTaskUpdates: List[GenericAttributesContainer] = ()
    deletedProxyTasks: List[BusinessObject] = ()
    serviceData: ServiceData = None
    newTasks: List[GenericAttributesContainer] = ()
    newDependencies: List[GenericAttributesContainer] = ()
    newAssignments: List[GenericAttributesContainer] = ()
    taskUpdates: List[GenericAttributesContainer] = ()
    dependencyUpdates: List[GenericAttributesContainer] = ()
    assignmentUpdates: List[GenericAttributesContainer] = ()
    deletedTasks: List[ScheduleTask] = ()
    deletedDependencies: List[TaskDependency] = ()



from __future__ import annotations

from tcsoa.gen.ImportExport._2017_11.FileImportExport import ExportToApplicationInputData3
from tcsoa.gen.ImportExport._2007_06.FileImportExport import ExportToApplicationResponse
from typing import List
from tcsoa.base import TcService


class FileImportExportService(TcService):

    @classmethod
    def exportToApplication(cls, inputs: List[ExportToApplicationInputData3]) -> ExportToApplicationResponse:
        """
        This operation is used for exporting Teamcenter objects to MSExcel. Each row in the MSExcel sheet can contain
        multiple Teamcener business objects and its properties. Teamcenter business objects should already be created
        so that the objects can be exported to MSExcel using the service operation.
        An MSExcel (.xlsm) file is generated at the server as a result of this operation. If the application format is
        "MSExcel" then the generated sheet is a static Excel spreadsheet. If the application format is "MSExcelLive"
        then the generated sheet is a Live Excel spreadsheet. "Live" sheet means that modifications done to the excel
        data will reflect to Teamcenter. If the export Option is "CheckOutObjects" then the objects will be checked out
        before exporting to Excel. Right now it only supports exporting to MSExcel, but we could support exporting to
        other application like MSWord in the future.
        
        Use cases:
        User can use this service operation to export 4GD objects from content pane to MSExcel. In the generated sheet,
        each row will contain multiple business objects and their properties. The properties can be edited from Excel
        and saved back to Teamcenter and vice versa.
        """
        return cls.execute_soa_method(
            method_name='exportToApplication',
            library='ImportExport',
            service_date='2017_11',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ExportToApplicationResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject, NamedVariantExpression, MEAppearancePathNode, BOMWindow, Item
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetABandABEInputInfo(TcBaseObj):
    """
    The 'GetABandABEInputInfo' structure represents the full set of inputs required for the operation
    'getApnComponents'(..).
    
    :var gcid: Specifies the generic component ID of the ABE to be searched.
    :var topArchTag: Specifies the top level Architecture Breakdown object.
    :var bomWindow: Specifies the BOMWindow object of parent architecture.
    """
    gcid: str = ''
    topArchTag: Item = None
    bomWindow: BOMWindow = None


@dataclass
class GetABandABEResponse(TcBaseObj):
    """
    The 'GetABandABEResponse' structure represents the response for the operation 'getApnComponents'(..) and has the
    reference to 'CompMeapnGcidAndDesc' object containing the generic component ID, description and MEAPN of the
    component. If there is any exception during the operation it will be added to the 'ServiceData' object and returned
    as a partial error.
    
    :var compsApnAndGcid: Reference to 'CompMeapnGcidAndDesc' object containing the generic component ID, description
    and MEAPN of the component.
    :var serviceData: An object of 'ServiceData' which contains updated data model objects and/or error objects with
    error messages.
    """
    compsApnAndGcid: List[CompMeapnGcidAndDesc] = ()
    serviceData: ServiceData = None


@dataclass
class GetAbeBomlineChildComponentsResponse(TcBaseObj):
    """
    This structure contains list of 'CompMeapnAndBomLine' and 'ServiceData'.
    
    :var componentsApnAndBomLines: It contains the list of 'CompMeapnAndBomLine' objects.
    
    :var serviceData: The 'ServiceData' containing full/partial errors.
    """
    componentsApnAndBomLines: List[CompMeapnAndBomLine] = ()
    serviceData: ServiceData = None


@dataclass
class GetAbeChildComponentsInputInfo(TcBaseObj):
    """
    This structure contains reference to Architecture Breakdown, BOMWindow and parent MEAppearancePathNode whose child
    MEAppearancePathNode objects needs to be retrieved.
    
    :var topLevelItem: Reference to Architecture Breakdown item, which is the topline in Architecture Breakdown
    Structure. If this reference is not supplied then operation will fail.
    
    :var meApn: MEAPN of parent whose child MEAPNs are to be retrieved. This is a mandatory parameter.
    
    :var bomWindow: Specifies the BOMWindow object of parent architecture. If it is not supplied then operation will
    fail.
    """
    topLevelItem: Item = None
    meApn: MEAppearancePathNode = None
    bomWindow: BOMWindow = None


@dataclass
class GetAbeMeapnChildComponentsResponse(TcBaseObj):
    """
    The 'GetAbeMeapnChildComponentsResponse' structure represents the response from the operation
    'getArchbreakdownMeapnChildComponents'(..) and has the reference to 'CompMeapnGcidAndDesc' objects containing the
    generic component ID, description and MEAPN of the child components. If there is any exception during the operation
    it will be added to the 'ServiceData' object and returned as a partial error.
    
    :var componentsApnAndGcid: Reference to 'CompMeapnGcidAndDesc' object containing the generic component ID,
    description and MEAPN of the component.
    :var serviceData: An object of 'ServiceData', returned as response for retrieving information on APNs, GCUID and
    description of child components, which contains updated data model objects and/or error objects with error messages.
    """
    componentsApnAndGcid: List[CompMeapnGcidAndDesc] = ()
    serviceData: ServiceData = None


@dataclass
class MetaExprTokens(TcBaseObj):
    """
    This structure contains the list of 'NVEMetaToken'. 'NVEMetaToken' represents a Token.
    
    :var tokens: Contains the list of 'NVEMetaToken'. 'NVEMetaToken' represents a token.
    """
    tokens: List[NVEMetaToken] = ()


@dataclass
class NVEMetaExpressionResponse(TcBaseObj):
    """
    This structure contains the list of MetaExprTokens and ServiceData.
    
    :var metaExprs: Contains the list of MetaExprTokens.
    :var serviceData: The ServiceData containing list of full/partial errors.
    """
    metaExprs: List[MetaExprTokens] = ()
    serviceData: ServiceData = None


@dataclass
class NVEMetaToken(TcBaseObj):
    """
    This structure represents a token.
    
    :var tokenType: It can take following values 
    -     tokenNull
    -     tokenAnd
    -     tokenOr
    -     tokenNot
    -     tokenBra
    -     tokenKet
    -     tokenNVE
    
    
    :var nveRef: Represents a NVE.
    """
    tokenType: TokenType = None
    nveRef: NVEReference = None


@dataclass
class NVEReference(TcBaseObj):
    """
    Structure representing a NamedVariantExpression (NVE). If 'TokenType' is not 'tokenNVE' then 'nveName' and 'nve'
    would be null.
    
    :var nveType: Type of NamedVariantExpression. It can take following values:
    -     nveAuthorized
    -     nveSplitting
    -     nveMandatorySplit
    
    
    :var nveName: Name of NVE. If 'TokenType' is not 'tokenNVE' then this field will be null.
    
    :var nve: Reference to the NVE object. If 'TokenType' is not 'tokenNVE' then this field will be null.
    """
    nveType: NVEType = None
    nveName: str = ''
    nve: NamedVariantExpression = None


@dataclass
class ValidateNVEMetaExprResponse(TcBaseObj):
    """
    This structure contains the list of 'XOChartData' and 'ServiceData' objects.
    
    :var xoCharts: List of the 'XOChartData' of NamedVariantExpression.
    :var serviceData: The 'ServiceData' containing full/partial errors.
    """
    xoCharts: List[XOChartData] = ()
    serviceData: ServiceData = None


@dataclass
class XOChartData(TcBaseObj):
    """
    This structure contains the XO chart information of the NamedVariantExpressions.
    
    :var noOfRows: Number of rows in the XO chart.
    :var noOfColumns: Number of column in the XO chart.
    :var colHeaderExprStrs: Name of the column header of the Expression in XO chart. 
    Example Color=BLUE.
    
    :var tableChars: The values for each cell in the XO chart. Each column can take  value as 1 (X)  or 0 (O).
    """
    noOfRows: int = 0
    noOfColumns: int = 0
    colHeaderExprStrs: List[str] = ()
    tableChars: List[int] = ()


@dataclass
class CompMeapnAndBomLine(TcBaseObj):
    """
    This structure contains MEAppearancePathNode and BOMLine.
    
    :var childMeapn: MEAppearancePathNode of child component.
    :var bomLine: BOMLine of the child component.
    """
    childMeapn: MEAppearancePathNode = None
    bomLine: BOMLine = None


@dataclass
class CompMeapnGcidAndDesc(TcBaseObj):
    """
    This structure contains the component information like generic component ID (gcid), component description and MEAPN.
    
    :var gcid: Specifies GCUID of child component.
    :var desc: Specifies description of child component.
    :var childMeapn: Specifies MEAPN of child component.
    """
    gcid: str = ''
    desc: str = ''
    childMeapn: MEAppearancePathNode = None


@dataclass
class AddDesignToProductResponse(TcBaseObj):
    """
    The 'AddDesignToProductResponse' structure represents the response for the operation 'addDesignToProduct'(..)
    containing the new PSOccurrence, Occurrence Type and Service data. If there is any exception during addition of
    part solutions it will be added to the 'ServiceData' object and returned as a partial error.
    
    :var newOcc: A collection of the occurrences created.
    :var newOccType: A collection of types of the corresponding occurrences created.
    :var serviceData: An object of 'ServiceData' which contains updated data model objects and/or error objects with
    error messages
    """
    newOcc: List[BusinessObject] = ()
    newOccType: List[BusinessObject] = ()
    serviceData: ServiceData = None


class NVEType(Enum):
    """
    Enumeration to classify NVE.
    """
    nveAuthorized = 'nveAuthorized'
    nveSplitting = 'nveSplitting'
    nveMandatorySplit = 'nveMandatorySplit'


class TokenType(Enum):
    """
    Enumeration to classify the Token.
    """
    tokenNull = 'tokenNull'
    tokenAnd = 'tokenAnd'
    tokenOr = 'tokenOr'
    tokenNot = 'tokenNot'
    tokenBra = 'tokenBra'
    tokenKet = 'tokenKet'
    tokenNVE = 'tokenNVE'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, Folder, Dataset, Item, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportToApplicationInputData(TcBaseObj):
    """
    The ExportToApplicationInputData structure represents all of the data necessary to export selected objects to
    Word/Excel.
    
    :var objectsToExport: The list of Teamcenter business objects to export.
    :var attributesToExport: The list of attributes to export.
    :var applicationFormat: The application format such as MSWordXML, MSExcel and MSExcelLive.
    Supported application formats for this operation
    - MSWordXML    This format is used for exporting Workspace objects to static MSWord application.
    - MSExcel    This format is used for exporting Teamcenter objects to static MSExcel  application.    
    - MSExcelLive    This format is used for exporting Teamcenter objects to Live MSExcel  application.
    
    
    :var templateId: The name of the MSWord/MSExcel template
    """
    objectsToExport: List[BusinessObject] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateId: str = ''


@dataclass
class ExportToApplicationResponse(TcBaseObj):
    """
    ExportToApplicationResponse structure represents the output of export to application operation. It contains the
    read ticket to the exported MSWord/MSExcel file.
    
    :var transientFileReadTickets: The transient file read tickets for the exported file.
    :var serviceData: The serviceData.
    """
    transientFileReadTickets: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ExtraObjectInfo(TcBaseObj):
    """
    The 'ExtraObjectInfo' structure represents additional object information that may be required to complete an
    operation. Example - It may be required by the client application to pass additional information about the relation
    (GRM) and properties on the relation to the server. This structure can be used to store the information about any
    relation objects.
    
    :var object: The tag of the Teamcenter Business object.
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var relationTypeName: The real type name of the relation.
    :var typeName: The real type name of the object.
    :var attrNameValuePairs: The vector of attributes names and its values to be set on the object.
    """
    object: BusinessObject = None
    clientId: str = ''
    relationTypeName: str = ''
    typeName: str = ''
    attrNameValuePairs: List[AttributeInfo] = ()


@dataclass
class ExtraObjectOutput(TcBaseObj):
    """
    This structure is defined to store information about the additional objects that are updated during the
    createOrUpdate operation.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var object: The tag of the Business object.
    """
    clientId: str = ''
    object: BusinessObject = None


@dataclass
class ExtraObjectOutputCAD(TcBaseObj):
    """
    This structure is defined to add additional objects as part of the response structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var object: The tag of the Business object.
    """
    clientId: str = ''
    object: BusinessObject = None


@dataclass
class GetContentInput(TcBaseObj):
    """
    The parameters required to open requirement in word.
    
    :var objectToProcess: Fulltext object, whose content to be  viewed.
    :var applicationFormat: The viewing application format. Only MSWordXML is supported.
    :var templateId: This parameter is not used currently.
    """
    objectToProcess: WorkspaceObject = None
    applicationFormat: str = ''
    templateId: str = ''


@dataclass
class GetRichContentResponse(TcBaseObj):
    """
    This structure holds FMS ticket of MSWord file generated as part of getRichContent operation.
    
    :var transientFileReadTickets: FMS ticket of word file that is generated as part of getRichContent operation.
    :var serviceData: The 'ServiceData'.
    """
    transientFileReadTickets: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ImportFromApplicationInputData(TcBaseObj):
    """
    The ImportFromApplicationInputData structure represents all of the data necessary to import an MSWord document into
    Teamcenter.
    
    :var transientFileWriteTicket: The file ticket of the. docx file to be imported into Teamcenter.
    :var applicationFormat: The supported application format is "MSWordXML"
    :var createSpecElementType: The subtype of SpecElement to be created.
    """
    transientFileWriteTicket: str = ''
    applicationFormat: str = ''
    createSpecElementType: str = ''


@dataclass
class ImportFromApplicationResponse(TcBaseObj):
    """
    ImportFromApplicationResponse structure represents the output of import from application operation. It contains the
    UID of the BOMWindow after the document is imported to Teamcenter.
    
    :var resultObjects: The resultant objects which contains the UID of the BOMWindow created after the document is
    imported.
    :var serviceData: The Service Data
    """
    resultObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class ItemInfo(TcBaseObj):
    """
    The 'ItemInfo' structure represents all of the data necessary to construct the Item object. It contains the
    information about the attributes that are set on the Item object. All attributes are passed as name/value pairs in
    the 'AttributeInfo' structure. The extraObject field contains information about the objects that are related to the
    newly created Item.
    
    :var item: The object tag of the object to be updated. If it is NULL then a new object will be created.
    :var itemId: The object Id (item_id) of the object to be updated. If it is NULL then a new ID is created.
    :var itemType: The object Type (object_type) of the object to be created or updated.
    :var name: The Name (object_name) of the object.
    :var description: The Description (object_desc) of the object.
    :var attrList: The attributes to be set on the object. This is in the form of map that stores attribute name and
    attribute value.
    :var extraObject: The structure containing information about the objects that are related to the Item.
    :var folder: The tag of the Folder under which newly created Item is to be pasted.
    """
    item: Item = None
    itemId: str = ''
    itemType: str = ''
    name: str = ''
    description: str = ''
    attrList: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    folder: Folder = None


@dataclass
class ItemRevInfo(TcBaseObj):
    """
    The 'ItemRevInfo' structure represents all of the data necessary to construct the ItemRevision object. It contains
    information about the attributes to be set on the revision object. All attributes are passed as name/value pairs in
    the 'AttributeInfo' structure. The extraObject field allows for the creation of an objects that will be related to
    this newly created ItemRevision.
    
    :var itemRevision: A tag of the ItemRevision to be created or updated. It is NULL for newly created Item.
    :var revId: The Id of the ItemRevision. It is NULL for new ItemRevision object.
    :var attrList: The attribute name and attribute value to be set on the ItemRevision.
    :var extraObject: The information about the objects that are related to the ItemRevision.
    """
    itemRevision: ItemRevision = None
    revId: str = ''
    attrList: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()


@dataclass
class NamedReferenceObjectInfo(TcBaseObj):
    """
    The 'NamedReferenceObjectInfo' structure represents all of the data necessary to construct the named reference file
    attached to a Dataset.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify 
    return data element and partial errors associated with this input structure.
    
    :var object: The reference to the named reference file for Dataset.
    :var relationTypeName: The real type name of the relation between the revision and the Dataset.
    
    :var typeName: The real type name of the Dataset.
    :var attrNameValuePairs: The attribute name-value pair to set on the Dataset.
    :var namedReferenceName: The real Dataset type.
    """
    clientId: str = ''
    object: BusinessObject = None
    relationTypeName: str = ''
    typeName: str = ''
    attrNameValuePairs: List[AttributeInfo] = ()
    namedReferenceName: str = ''


@dataclass
class AttributeInfo(TcBaseObj):
    """
    This structure allows the caller to create or update named attributes. This structure must contain the name of the
    attribute and the value of the attribute to set.
    
    :var name: The name of the attribute to set.
    :var value: The value of the attribute to set.
    """
    name: str = ''
    value: str = ''


@dataclass
class AttributeInfoCAD(TcBaseObj):
    """
    This structure allows the caller to create or update named attributes. This structure must contain the name of the
    attribute and the value of the attribute to set.
    
    :var name: The name of the attribute to set.
    :var value: The value of the attribute to set.
    """
    name: str = ''
    value: str = ''


@dataclass
class PartInfo(TcBaseObj):
    """
    The data required to create an Item, ItemRevision and one or more Dataset.  If the Item already exists then it also
    has the information required to be updated for that Item.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return 
    data elements and partial errors associated with this input structure.
    :var itemInput: The information required to create or update an Item.
    :var itemRevInput: The information required to create or update an ItemRevision.
    :var datasetInput: The information required to create or update one or more Dataset objects.
    """
    clientId: str = ''
    itemInput: ItemInfo = None
    itemRevInput: ItemRevInfo = None
    datasetInput: List[DatasetInfo] = ()


@dataclass
class SetContentInput(TcBaseObj):
    """
    'SetContentInput' structure represents the parameters required to set the contents to Fulltext object.
    
    :var objectToProcess: Fulltext object.
    :var transientFileWriteTicket: FMS ticket of Word file.
    """
    objectToProcess: WorkspaceObject = None
    transientFileWriteTicket: str = ''


@dataclass
class SetRichContentResponse(TcBaseObj):
    """
    'SetRichContentResponse' - structure represents response parameters of setRichContent SOA.
    
    :var resultObjects: This parameter is not used.
    :var serviceData: The Service Data.
    """
    resultObjects: List[WorkspaceObject] = ()
    serviceData: ServiceData = None


@dataclass
class CommitDatasetFileInfoCAD(TcBaseObj):
    """
    This structure has information to commit named reference files to input Dataset. This structure contains the
    necessary  information like the file ticket and if a new version of Dataset is to be created.
    
    :var dataset: A tag of the Dataset to be updated with the named reference file.
    :var createNewVersion: The value of true means that a new Dataset to be created, value of false means that the
    existing Dataset to be updated with the named reference file.
    :var datasetFileTicketInfos: The information about the Dataset file tickets.
    """
    dataset: Dataset = None
    createNewVersion: bool = False
    datasetFileTicketInfos: List[DatasetFileTicketInfoCAD] = ()


@dataclass
class CreateOrUpdateOutput(TcBaseObj):
    """
    This structure is defined to store information about the newly created Dataset or updates to the existing Dataset.
    It contains all the information about the related objects that user needs to pass during the creation or update of
    Dataset.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var item: The tag of the Item that is created or updated.
    :var itemRev: The tag of the ItemRevision that is created or updated.
    :var datasetOutput: The tag of the Dataset that is created or updated.
    :var extraItemObjs: The structure containing information about the additional Item objects created.
    :var extraItemRevObjs: The structure containing information about the additional ItemRevision objects created.
    """
    clientId: str = ''
    item: Item = None
    itemRev: ItemRevision = None
    datasetOutput: List[DatasetOutput] = ()
    extraItemObjs: List[ExtraObjectOutput] = ()
    extraItemRevObjs: List[ExtraObjectOutputCAD] = ()


@dataclass
class CreateOrUpdateResponse(TcBaseObj):
    """
    This structure represents the output of createOrUpdate operation.  It has the information about the newly created
    Dataset or update to existing Dataset.
    
    :var output: The list of structures containing information about the Item,
    ItemRevision or the Dataset to be created or updated.
    :var serviceData: The serviceData.
    """
    output: List[CreateOrUpdateOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetFileInfoCAD(TcBaseObj):
    """
    This structure allows the user to create or update the dataset with the named attributes information. This
    structure must contain either an "AttributesToSet" element or an "AttributesToUnset" element, but not both.
    
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var fileName: The name of the named reference file to be updated.
    :var namedReferencedName: The named references string for that object.
    :var isText: Flag to indicate if it is text data. Value of true indicates it is text data. Value of false indicates
    binary data.
    :var allowReplace: Flag to indicate whether to replace the data. Value of true indicates data to be overwritten.
    Value of false indicates data will not be overwritten.
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    isText: bool = False
    allowReplace: bool = False


@dataclass
class DatasetFileTicketInfoCAD(TcBaseObj):
    """
    This structure has Dataset FMS file ticket information.
    
    :var datasetFileInfo: The Dataset file ticket information.
    :var ticket: The name of the ticket.
    """
    datasetFileInfo: DatasetFileInfoCAD = None
    ticket: str = ''


@dataclass
class DatasetInfo(TcBaseObj):
    """
    The DatasetInfo structure represents all of the data necessary to construct the Dataset object. All the attributes
    required to be set on the Dataset are passed as name/value pairs in the 'AttributeInfo' structure. The extraObject
    field allows for the creation of an object(s) that will be related to this newly created Dataset.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var dataset: The tag of Dataset.
    :var mappingAttributes: The mapping attributes to be set on the Dataset.
    :var extraObject: Additional objects to be created and related to the Dataset.
    :var datasetFileInfos: The information about files attached to the Dataset.
    :var namedReferenceObjectInfos: The information about objects attached to the Dataset.
    :var name: The name of Dataset.
    :var description: The description for Dataset.
    :var type: The real type name of Dataset.
    :var id: The id of Dataset.
    :var datasetRev: The name of Dataset revision.
    :var itemRevRelationName: The relation name of the revision to Dataset.
    :var createNewVersion: The flag to create new version. Value of true indicates new revision to be created, value of
    false indicates no new revision to be created.
    :var attrList: The attributes to be set on the Dataset.
    """
    clientId: str = ''
    dataset: Dataset = None
    mappingAttributes: List[AttributeInfoCAD] = ()
    extraObject: List[ExtraObjectInfo] = ()
    datasetFileInfos: List[DatasetFileInfoCAD] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo] = ()
    name: str = ''
    description: str = ''
    type: str = ''
    id: str = ''
    datasetRev: str = ''
    itemRevRelationName: str = ''
    createNewVersion: bool = False
    attrList: List[AttributeInfo] = ()


@dataclass
class DatasetOutput(TcBaseObj):
    """
    This structure is defined to store information about the output Dataset objects. It also contains information about
    the additional objects that are related to the Dataset.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var dataset: The tag of the Dataset.
    :var commitInfo: The structure containing information to commit named reference files.
    :var extraObjs: The structure containing information about the objects that are related to the Dataset.
    :var namedRefObjs: The structure containing information about the additional named reference objects
    """
    clientId: str = ''
    dataset: Dataset = None
    commitInfo: List[CommitDatasetFileInfoCAD] = ()
    extraObjs: List[ExtraObjectOutput] = ()
    namedRefObjs: List[ExtraObjectOutputCAD] = ()



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssemblyConfigurationResponse(TcBaseObj):
    """
    This structure is used by SOA Teamcenter::Soa::Cad::_2018_06::
    StructureManagement::writeAssemblyConfigurationDetails for returning XML output file name in
    readFileTicketForStructureData and status of operation is written to serviceData.
    
    :var readFileTicketForStructureData: This is the FMS read file ticket for the TC XML data file written to the
    transient volume. The TC XML file can be downloaded using this ticket.
    :var serviceData: This service data contains any partial errors which may have been encountered during processing.
    The partial error client Ids match the client Id for the input which failed.
    """
    readFileTicketForStructureData: str = ''
    serviceData: ServiceData = None



from tcsoa.gen.Internal_Query._2012_02.services import SavedQueryService as imp0
from tcsoa.gen.Internal_Query._2013_05.services import SavedQueryService as imp1
from tcsoa.gen.Internal_Query._2014_10.services import SavedQueryService as imp2
from tcsoa.gen.Internal_Query._2008_06.services import FinderService as imp3
from tcsoa.base import TcService


class SavedQueryService(TcService):
    describeSavedQueryDefinitions = imp0.describeSavedQueryDefinitions
    describeSavedQueryDefinitions2 = imp1.describeSavedQueryDefinitions2
    describeSavedQueryDefinitions3 = imp2.describeSavedQueryDefinitions3


class FinderService(TcService):
    findObjectsByClassAndAttributes = imp3.findObjectsByClassAndAttributes
